package apollo.ares;

import javax.media.*;
import javax.media.protocol.*;
import java.io.*;
import apollo.hermes.jini.*;
import apollo.demeter.client.*;
import apollo.demeter.save.*;
import apollo.common.movie.*;
import apollo.common.*;
import apollo.common.gui.*;
import apollo.common.jini.*;
import apollo.common.save.*;
import apollo.common.tui.TextLog;
import apollo.ares.gui.*;
import apollo.ares.media.*;
import java.rmi.*;
import java.rmi.server.*;
import net.jini.lease.*;
import net.jini.core.lookup.*;
import net.jini.discovery.*;
import net.jini.core.discovery.*;
import net.jini.core.event.*;
import net.jini.core.lease.*;
import net.jini.core.lookup.ServiceItem;
/**
 *
 * @author Konrad Rzeszutek (konrad@darnok.org)
 * @version $Id: Ares.java,v 1.5 2002/04/28 19:14:11 uid500 Exp $
 */
public class Ares implements DiscoveryListener, Daemon{

  private final static String DAEMON_NAME  = "Ares";
  private final static String DAEMON_VER   = "$Id: Ares.java,v 1.5 2002/04/28 19:14:11 uid500 Exp $";

  private String ourname;
  private String ourhost;
  private MainFrame frame;
  private ServerFinder finder;
  private LookupDiscovery lookup              = null;
    //looks for all available lookup servers
  private Log log = new TextLog();

  private CameraProcessor camera;
  private MotionProcessor motion;
  private DemeterClient c;
  private Saver jpg_save;
  private MovieCreator movie;

  public Ares(boolean nogui,String DIR, String CAMERA) {

    // Find the camera
    getHostInfo();
    // Start the GUI
    if (!nogui) {
      frame = new MainFrame(DAEMON_NAME, this);
      frame.setSize(800,800);
      frame.validate();
      frame.setVisible(true);
      // Wait until its up?

      Object waitFrame = new Object();
      while (frame.log == null) {
        synchronized (waitFrame) {
          try {
            waitFrame.wait(1000);
          } catch (InterruptedException e) { }
        }
      }
      log = ((Log)frame.log);
    }

    // Start the CameraProcessor

    camera = new CameraProcessor(log);

      try {

        camera.open(new MediaLocator(CAMERA));
    // Clone it.
        DataSource ds = camera.getDataOutput();

        ds = Manager.createCloneableDataSource(ds);


          /*
          Note about SourceClonalbe. If not used, the last ds will
          control and the other ones won't ever get any data.
          */
        // Start the MotionProcessor

        MotionDetectionEffect effect = new MotionDetectionEffect();
        MotionControl motionF = new MotionControl(effect);

        motion = new MotionProcessor(log,effect);
        motion.open(ds);

        // Clone it as well.

        DataSource motion_ds = motion.getDataOutput();
        motion_ds = Manager.createCloneableDataSource(motion_ds);

        // Start up our client.
        c = new DemeterClient(ourhost,ourname,DIR);

        // Create the directories if they don't exist.
        createDirectory(c.getImageDir());
        createDirectory(c.getMovieDir());

        // Init our JPGsaver.
        Codec[] codecs = new Codec[1];
        codecs[0] = new TimeStampEffect(ourhost);
        jpg_save = new RTPtoJPG(log,c, codecs);
        jpg_save.open(motion_ds);

        // Start up Movie Creator
         movie = new MovieCreator(log);
         movie.MOVIE_CREATION_TIME = 5;
         movie.register(c);

        if (!nogui) {
                // Hook up CameraProcessor to VideoPlayer

          VideoPlayer realtimeP = new VideoPlayer("Real time camera feed",
            ((SourceCloneable)ds).createClone());
                  // Hook up MotionProcessor to VideoPlayer
          VideoPlayer motionP = new VideoPlayer("Motion time camera feed",
            ((SourceCloneable)motion_ds).createClone());

          frame.addJInternalFrame(realtimeP);
          frame.addJInternalFrame(motionP);
          motionP.setLocation(realtimeP.getX()+motionP.getWidth()+30,
            realtimeP.getY());
          frame.addJInternalFrame(motionF);
          motionF.setLocation(realtimeP.getX()+motionP.getWidth()+30,
             realtimeP.getY()+motionP.getHeight()+30);
          frame.addJInternalFrame(((DemeterClient)c).getFrame());
          c.getFrame().setLocation(realtimeP.getX(),realtimeP.getY()+
            motionP.getHeight()+30);
      }


      finder = new ServerFinder(log, c, motion_ds);

      } catch (Exception e) {
        e.printStackTrace();
        if (log.fatalError(e.getMessage())) exitDaemon();
      }
    // Start the Jini/RMI finder

    // Now add the Jini listener for the Jini services offered and call the RMI
    //this.leaser = new LeaseRenewalManager( );
    //System.setSecurityManager(new RMISecurityManager( ));
    try{
      lookup = new LookupDiscovery(LookupDiscovery.ALL_GROUPS);
    }catch(java.io.IOException e){
        e.printStackTrace();
        if (log.fatalError(e.getMessage())) exitDaemon();
    }

    lookup.addDiscoveryListener(this);


  }

  /**
   * Find out what our IP is. Where the RMI server resides, and what port
   * is the RMI server up on. These info are assumed to be the local machine,
   * but if the RMI server is located somewhere else, the
   *  RMISERVER variable and RMIPORT hold the values.
   */
  public void getHostInfo() {


      ourname = System.getProperty("ARESNAME","Default");

      try {
        ourhost = java.net.InetAddress.getLocalHost().getHostAddress();

      } catch (Exception e) {

        if (log.fatalError(e.getMessage())) exitDaemon();
      }

  }
  /**
   * Called when one or more lookup service registrars has been discovered.
   * The method should return quickly; e.g., it should not make remote calls.
   *
   * @param e - the event that describes the discovered registrars
   */
  public synchronized void discovered(DiscoveryEvent e){

    ServiceTemplate hermes = new ServiceTemplate(null,
      new Class[]{JiniHermesInfo.class}, null);
    ServiceRegistrar[ ] registrars = e.getRegistrars( );

    for(int i =0; i< registrars.length; i++){

      try{
          ServiceRegistrar registrar = registrars[i];
          log.println("Discovered "+registrar.getLocator().getHost() +
            " Jini server!","Jini");
          checkJiniFor(hermes,  registrars[i]);
      }catch(Exception ex){
          log.normalError(ex.getMessage());
        ex.printStackTrace();

	    }//end catch
	}//end outer for
    }//end method discovered

  /**
   * Checks the Jini for the <code>template</code> type server with the
   * given <code>registrar</code> and registers a JiniListner in case
   * the service appears at some point later on.
   *
   */
    public void checkJiniFor(ServiceTemplate template,ServiceRegistrar registrar)
        throws RemoteException{

        ServiceMatches services = registrar.lookup(template,Integer.MAX_VALUE);

        LookupLocator locator = registrar.getLocator( );

	/**
	 *  get the information from each one of the services
	 */
	for(int j =0; j< services.items.length; j++){
	  if(services.items[j].service != null){
                JiniServerComponentInfo info =
                  (JiniServerComponentInfo)services.items[j].service;
                finder.newRegistrar(info);
		}//end if
	    }//end for
          EventRegistration ereg = registrar.notify(template,
            ServiceRegistrar.TRANSITION_NOMATCH_MATCH,
	    new JiniListner(template, finder), // Hmm for
	    null,
            Lease.ANY);

    }

    /**
     * Called when one or more lookup service registrars has been discarded.
     *
     * @param e - the event that describes the discarded registrars
     */
    public synchronized void discarded(DiscoveryEvent e){
	ServiceRegistrar[] registrars = e.getRegistrars( );
	for(int i=0; i<registrars.length;i++){
	/**
	 * a condition where a Service Registrar
	 * leaves the vicinity.We cannot move the
	 * users from the lookup server becuase they
	 * may actually be present on the other
	 * lookup servers.
	 */
         try{
 		ServiceRegistrar registrar = registrars[i];
		LookupLocator locator = registrar.getLocator( );
		log.println("The Lookup Service "+locator.getHost( )+
                  " has left.","Jini");
	    }catch(Exception ex){
		//System.out.println("Registrar could not leave");
                //ex.printStackTrace();
                log.println("Registrar could not leave","Jini");
	    }
	}//end for
    }//end method

  public void exitDaemon() {

    // Terminate everything.
    camera.stop();
    camera.close();
    camera = null;
    motion.stop();
    motion.close();
    motion = null;
    //jpg_save.stop();
    jpg_save.close();
    jpg_save = null;

    movie.flush(c);
    movie.remove(c);
    movie.closeAll();
    movie.killThread();
    movie = null;

    frame = null;

    System.exit(0);

  }

  /**
   * Helper method used to create directories
   *
   */
  private void createDirectory (String dir) {

          /*
        Create the directory.

        */

        File[] dirs = null;
        File top = new File(dir);
        String step = top.getAbsolutePath();
        int cnt = 0;

        for (int i = 0; i< step.length(); i++)
          if (step.charAt(i) == '\\') cnt++;

        //System.out.println(step+","+cnt);
        dirs = new File[cnt+1];
        dirs[dirs.length-1] = top;
        for (int i = dirs.length-2; i>=0; i--)
          dirs[i] = new File(dirs[i+1].getParent());

        for (int i = 0; i< dirs.length; i++) {
          //System.out.println(dirs[i].getAbsolutePath());
          dirs[i].mkdir();
        }
  }
  private static void prUsage() {

        System.out.println("java -jar ares [-nogui] -d [directory] -c [camera]");
        System.exit(-2);

  }
  public static void main(String[] args) {

    int i = 0;
    boolean nogui = false;
    String dir = null;
    String cam = null;
    while (i < args.length) {
      if (args[i].equals("-nogui")) {
		i++;
                nogui = true;
      }
      else if (args[i].equals("-h")) {
          prUsage();
      }
      else if (args[i].equals("-d")) {
          i++;
          if (i >= args.length)
            prUsage();
          dir = args[i];
      }
      else if (args[i].equals("-c")) {
          i++;
          if (i >= args.length)
            prUsage();
          cam = args[i];
      }

      i++;
    }

    if ((dir == null) ||  (cam == null))
      prUsage();
    Ares a = new Ares(nogui, dir,cam);
  }
}
