package apollo.ares;

import apollo.common.*;
import apollo.common.tui.*;
import apollo.common.events.*;
import java.awt.Component;
import java.io.*;
import javax.media.*;
import javax.media.format.*;
import javax.media.protocol.*;
import javax.media.protocol.DataSource;
import javax.media.datasink.*;
/**
 * @author Konrad Rzeszutek (konrad@darnok.org)
 * @version $Id: CameraProcessor.java,v 1.4 2002/04/25 18:59:01 uid500 Exp $
 */

public class CameraProcessor implements ControllerListener {

    private Processor p;
    private DataSource ds;
    private Object waitSync = new Object();
    private boolean stateTransitionOK = true;
    private Log log = null;

    public CameraProcessor(Log log) {

      this.log = log;
    }

    /**
     * Given a datasource, create a processor and use that processor
     * as a player to playback the media.
     *
     *
     */
    public void open(MediaLocator ds) throws NoProcessorException,
      UnknownProblemException, IOException,
      IncompatibleSourceException, UnsupportedFormatException {


        p = Manager.createProcessor(ds);
	p.addControllerListener(this);
        log.println("Created processor",Log.INFO);

	// Put the Processor into configured state.
	p.configure();
	if (!waitForState(p.Configured)) {
            throw new UnknownProblemException("Failed to configure the processor.");
	}

	// So the output can be cloned.
        p.setContentDescriptor(new ContentDescriptor(FileTypeDescriptor.MIXED));
	// Realize the processor.
        //log.println("Realizing processor",Log.INFO);
	p.prefetch();

	if (!waitForState(p.Prefetched)) {
          throw new UnknownProblemException("Failed to prefetch the processor.");
	}
        log.println("Camera feed prefetched.",Log.INFO);

        log.println("Starting camera feed.", Log.INFO);
	p.start();

    }

    public Component getVisualComponenet() {

      if (p != null)
        return p.getVisualComponent();
      return null;
    }
    public Component getControlPanelComponent() {

      if (p != null)
        return p.getControlPanelComponent();
      return null;
    }
    public void start() {
      p.start();
    }

    public void close() {

      p.close();
      p = null;
      System.gc();
    }
    public void stop() {

      p.stop();
    }

    public DataSource getDataOutput() {

      if (p != null)
        return p.getDataOutput();
      return null;
    }

    /**
     * Block until the processor has transitioned to the given state.
     * Return false if the transition failed.
     */
    private boolean waitForState(int state) {
	synchronized (waitSync) {
	    try {
		while (p.getState() != state && stateTransitionOK)
		    waitSync.wait();
	    } catch (Exception e) {}
	}
	return stateTransitionOK;
      }

    /**
     * Controller Listener.
     */
    public void controllerUpdate(ControllerEvent evt) {

        log.println(evt.toString(),this.getClass().getName());
	if (evt instanceof ConfigureCompleteEvent ||
	    evt instanceof RealizeCompleteEvent ||
	    evt instanceof PrefetchCompleteEvent) {
	    synchronized (waitSync) {
		stateTransitionOK = true;
		waitSync.notifyAll();
	    }
	} else if (evt instanceof ResourceUnavailableEvent) {
	    synchronized (waitSync) {
		stateTransitionOK = false;
		waitSync.notifyAll();
	    }
	} else if (evt instanceof EndOfMediaEvent) {
	    p.close();
	    //System.exit(0);
	}
    }
}