package apollo.ares;
import apollo.common.*;
import apollo.common.tui.*;
import apollo.common.events.*;
import java.awt.Component;
import java.io.*;
import javax.media.*;
import javax.media.format.*;
import javax.media.protocol.*;
import javax.media.protocol.DataSource;
import javax.media.datasink.*;
import javax.media.control.TrackControl;
/**

 * @author Konrad Rzeszutek (konrad@darnok.org)
 * @version $Id: MotionProcessor.java,v 1.3 2002/04/28 19:14:11 uid500 Exp $
 */
public class MotionProcessor implements ControllerListener {

    private Processor p;
    private DataSource ds;
    private Object waitSync = new Object();
    private boolean stateTransitionOK = true;
    private Log log = null;
    private Effect effect;
    public MotionProcessor(Log log,Effect effect) {

      this.log = log;
      this.effect = effect;
    }

    /**
     * Given a datasource, create a processor and use that processor
     * as a player to playback the media.
     *
     *
     */
    public void open(DataSource ds) throws NoProcessorException,
      UnknownProblemException, IOException, UnsupportedPlugInException,
      IncompatibleSourceException, UnsupportedFormatException {

        this.ds = ds;
        p = Manager.createProcessor(ds);

	p.addControllerListener(this);


	// Put the Processor into configured state.
	p.configure();
	if (!waitForState(p.Configured)) {
            throw new UnknownProblemException("Failed to configure the processor.");
	}

	// So the output can be cloned.
        p.setContentDescriptor(new ContentDescriptor(FileTypeDescriptor.MIXED));

        // Add the effect codec.
        TrackControl tc[] = p.getTrackControls();
	TrackControl videoTrack = null;

	for (int i = 0; i < tc.length; i++) {
	    if (tc[i].getFormat() instanceof VideoFormat) {
		videoTrack = tc[i];
		break;
	    }
	}

	if (videoTrack == null) {
          throw new UnknownProblemException("Input datasource does not contain"+
            " any video tracks.");
        }
        Codec codec[] = {  effect};
        videoTrack.setCodecChain(codec);
	// Realize the processor.
	p.prefetch();

	if (!waitForState(p.Prefetched)) {
          throw new UnknownProblemException("Failed to prefetch the processor.");
	}

	p.start();

    }

    public void stop() {
      p.stop();
    }
    public void close() {
      p.close();
      p = null;
      System.gc();


    }

    public Component getVisualComponenet() {
      if (p != null)
        return p.getVisualComponent();
      return null;
    }
    public Component getControlPanelComponent() {

      if (p != null)
        return p.getControlPanelComponent();
      return null;
    }
    public DataSource getDataOutput() {

      if (p != null)
        return p.getDataOutput();
      return null;
    }

    /**
     * Block until the processor has transitioned to the given state.
     * Return false if the transition failed.
     */
    private boolean waitForState(int state) {
	synchronized (waitSync) {
	    try {
		while (p.getState() != state && stateTransitionOK)
		    waitSync.wait();
	    } catch (Exception e) {}
	}
	return stateTransitionOK;
      }

    /**
     * Controller Listener.
     */
    public void controllerUpdate(ControllerEvent evt) {

        log.println(evt.toString(), this.getClass().getName());
	if (evt instanceof ConfigureCompleteEvent ||
	    evt instanceof RealizeCompleteEvent ||
	    evt instanceof PrefetchCompleteEvent) {
	    synchronized (waitSync) {
		stateTransitionOK = true;
		waitSync.notifyAll();
	    }
	} else if (evt instanceof ResourceUnavailableEvent) {
	    synchronized (waitSync) {
		stateTransitionOK = false;
		waitSync.notifyAll();
	    }
	} else if (evt instanceof EndOfMediaEvent) {
	    p.close();
	    //System.exit(0);
	}
    }
}