package apollo.ares;
import javax.media.protocol.*;
import javax.media.*;
import javax.media.format.*;
import java.net.MalformedURLException;
import java.io.IOException;
import apollo.common.*;
import apollo.common.events.*;
import apollo.common.client.*;
import apollo.common.jini.JiniServerComponentInfo;
import apollo.demeter.client.*;
import apollo.demeter.rmi.*;
import apollo.hermes.rmi.*;
import apollo.nemesis.rmi.*;
import java.rmi.*;
import java.rmi.server.*;
import apollo.ares.media.*;

/**
 * @author Konrad Rzeszutek (konrad@darnok.org)
 * @version $Id: ServerFinder.java,v 1.3 2002/04/28 19:14:12 uid500 Exp $
 */

public class ServerFinder extends Thread{

  private Log log;
  private DemeterClient c;
  private Client c_send;
  private DataSource ds;

  private MediaLocator rtp;
  private RTPProcessor rtpProc;
  private DataSink rtpSink;
  private String nemesis;
  private String demeter;
  private String hermes;
  private boolean isDemeterOk = false;
  private boolean isNemesisOk = false;
  private Object waitThread = new Object();
  private Object waitTime = new Object();
  private boolean stop = false;

  public ServerFinder(Log log, DemeterClient c, DataSource ds) throws
    NoProcessorException, UnknownProblemException,UnsupportedPlugInException,
    IOException, IncompatibleSourceException, UnsupportedFormatException{

    this.log = log;
    this.c = c;
    this.ds = ds;
    c_send = new Client(c.getHost(),c.getName());
    rtpProc = new RTPProcessor(log);
    rtpProc.open(((SourceCloneable)ds).createClone());
    this.start();
  }
  /**
   * Connect to Hermes and find the server for us, by the <code>type</code>
   * given to us.
   *
   * @param type which has to be <code>JiniServerComponentInfo.DEMETER</code> or
   *  <code>JiniServerComponentInfo.MORS</code> or <code>JiniServerComponentInfo.NEMESIS</code>
   *  or <code>JiniServerComponentInfo.HERMES</code>.
   *
   * @return null if no server could been found, otherwise an RMI type name: rmi://<host>/<binding name>
   *
   */
  private String findServer(String type) throws RemoteException,
    MalformedURLException, NotBoundException{


      String result = null;

      if (hermes != null) {
        HermesRMIInterface info = (HermesRMIInterface)Naming.lookup(hermes);
        result = info.getURLForServerType(type);
      }

      return result;

  }
  private void registerDemeter() throws RemoteException, MalformedURLException,
   NotBoundException {

    // Register with Demeter, our RTP session
    DemeterRMIInterface rmi_d = (DemeterRMIInterface)Naming.lookup(demeter);
    rmi_d.openRTP(rtp);
    log.println("Registered with Demeter ("+demeter+")",Log.INFO);
    isDemeterOk = true;
  }
  private void registerNemesis() throws RemoteException, MalformedURLException,
     NotBoundException{



    // Now we connect to Nemesis_Hermes end and get the right info?
    //ServerRMIInfoInterface info = (ServerRMIInfoInterface)Naming.lookup(nemesis);

    NemesisRMIInterface rmi_n = (NemesisRMIInterface)Naming.lookup(nemesis);

    rmi_n.register(rtp, c_send);
    log.println("Registered with Nemesis ("+nemesis+")",Log.INFO);
    isNemesisOk = true;

  }
  private void unregisterDemeter() {

    try {
      DemeterRMIInterface rmi_d = (DemeterRMIInterface)Naming.lookup(demeter);
      rmi_d.closeRTP(rtp);
      log.println("Unregistered with Demeter ("+demeter+")",Log.INFO);
    } catch (Exception e) { }
    demeter = null;
    // Wait 10 seconds.

    synchronized (waitTime) {
      try {
        waitTime.wait(10000);
      } catch (InterruptedException e) { }
    }

  }
  private void unregisterNemesis() {


    try {
      NemesisRMIInterface rmi_n = (NemesisRMIInterface)Naming.lookup(nemesis);
      rmi_n.unregister(rtp, c_send);
      log.println("Unregistered with Nemesis ("+nemesis+")",Log.INFO);
    } catch (Exception e) {}

    //Clean the string variables?

    nemesis = null;
    synchronized (waitTime) {
      try {
        waitTime.wait(10000);
      } catch (InterruptedException e) { }
    }

  }
  private void connectionCheck() {

  try {

    // Check _all_ connection
    NemesisRMIInterface rmi_d = (NemesisRMIInterface)Naming.lookup(nemesis);
    rmi_d.isConnectionOk();
    //log.println("Nemesis is ok",Log.INFO);
  } catch (Exception e) { nemesis = null; isNemesisOk = false;}

    try {
    // Check the connection to the specified hosts... how?
    // Check _all_ connection
    DemeterRMIInterface rmi_d = (DemeterRMIInterface)Naming.lookup(demeter);
    rmi_d.isConnectionOk();
    //log.println("Demeter is Ok",Log.INFO);
  } catch (Exception e) { demeter = null; isDemeterOk = false;}
  try {

    HermesRMIInterface info = (HermesRMIInterface)Naming.lookup(hermes);
    info.isConnectionOk();

  } catch (Exception e) {
    log.println("Hermes is not online.("+e.getMessage()+")",Log.ERROR);
    hermes = null; }

  }

  private void startRTP() throws MalformedURLException, NoDataSinkException, IOException{

    //MediaLocator rtpURL = new MediaLocator();

    rtpSink  = Manager.createDataSink(rtpProc.getDataOutput(), rtp);
    rtpSink.open();
    rtpSink.start();
  }

  private void closeRTP() throws IOException{

    rtpSink.stop();
    rtpSink.close();

    rtpSink = null;
    System.gc();
  }

  public  synchronized void newRegistrar(JiniServerComponentInfo i) {

    // Means we found a Hermes - our load balancer. Query it for differnt types
    if (hermes != null)
      connectionCheck();

    try {
    if (hermes == null) /* We never used hermes */
      log.println("Finding Hermes ",Log.INFO);
      hermes = "rmi://"+i.getRMIHost()+":"+i.getRMIPort()+"/"+i.getRMIName();
    } catch (Exception e) {

      //
      //hermes = null;
    }

    if (rtp == null)  {

     try {
      log.println("Finding Hermes ("+hermes+")",Log.INFO);
      // Never started the RTP connection.
       HermesRMIInterface info = (HermesRMIInterface)Naming.lookup(hermes);
       String rtpS = info.getRTPURL();
       log.println("Starting RTP to :"+rtpS,Log.INFO);
       rtp = new MediaLocator(rtpS);

     } catch (Exception e) {
      log.println("Hermes is not online.("+e.getMessage()+")",Log.ERROR);
    }
      // Start multicasting..
     try {
      if (rtp != null)
       startRTP();
      } catch (Exception e) {
         log.normalError("Couldn't start transmitting data.\n"+e.getMessage());}
    }


    // Should this be somewhere else?
    try {
    if (!isNemesisOk) {

      log.println("Finding Nemesis [last address: "+nemesis+"]",Log.INFO);
      if (nemesis != null) { /* We still have a connection */
        unregisterNemesis();
      }
      nemesis = findServer(JiniServerComponentInfo.NEMESIS);
      if (!nemesis.equals("UNKNOWN")) {
        nemesis += NemesisRMIInterface.NEMESIS_RMI;
      // We still have our RTP session in progress, therefore no need to
      // restart the RTP session.
        registerNemesis();
      }
    }
    } catch (Exception e) {
      //e.printStackTrace();
      log.println("Couldn't find Nemesis!\n"+e.getMessage(),Log.ERROR);
    }

    try {
    if (!isDemeterOk) {
      log.println("Finding Demeter [last address: "+demeter+"]",Log.INFO);
      if (demeter != null) { /* We still have a connection */
        unregisterDemeter();
      }
      demeter = findServer(JiniServerComponentInfo.DEMETER);
      if (!demeter.equals("UNKNOWN")) {
        demeter += DemeterRMIInterface.DEMETER_RMI;
      // We still have our RTP session in progress, therefore no need to
      // restart the RTP session.
        registerDemeter();
      }
    }
   } catch (Exception e) {
      log.println("Couldn't find Demeter!\n"+e.getMessage(),Log.ERROR);
    }


  }
  public void run() {
    while(!stop) {
      //log.println("ServerFinder:run: Woke up.. checking ["+hermes+"]",Log.DEBUG);
      if (hermes != null)
        newRegistrar(null);

      synchronized (waitThread) {
        try {
          waitThread.wait(10000);
        } catch (InterruptedException e) { }
      }

    }
  }
}