package apollo.common;

/**
 *
 * $Log: FaceDetectionEvent.java,v $
 * Revision 1.4  2002/04/16 19:21:27  uid500
 * Fixed RMI timeouts
 *
 * Revision 1.3  2002/04/14 01:25:12  uid500
 * Provided interface for daemons
 *
 * Revision 1.2  2002/03/31 03:22:08  uid500
 * *** empty log message ***
 *
 * Revision 1.1.1.1  2002/03/30 18:51:33  uid500
 * Apollo src code import
 *
 * @author Konrad Rzeszutek (konrad@darnok.org)
 * @version $Id: FaceDetectionEvent.java,v 1.4 2002/04/16 19:21:27 uid500 Exp $
 */

import java.io.Serializable;
import java.util.EventObject;

/**
 * Face Detection event - generated when a face is matched.
 */
public class FaceDetectionEvent extends EventObject implements Serializable {

  private byte[] camera_img;
  private byte[] matched_img;
  private String host;
  private String location;
  private String date;
  private String desc;

  /**
   * Face Detection Event object.
   *
   * @param src Source of the disptch of this event.
   */
  public FaceDetectionEvent (Object src) {

    super(src);
  }
  /**
   * Initialize the Event
   *
   * @param src The object which generated this event.
   * @param h The host where the Event was generated.
   * @param l The String location describing the host where the Event
   *  was generated.
   * @param d The time and date
   * @param desc Description explaining the event.
   * @param c Image captured from the camera
   * @param m Image matching the captured image (as determined by the face
   *  recognition algorhytm).
   */
  public FaceDetectionEvent(Object src, String h, String l, String d, String desc, byte[] c, byte[] m) {

    super(src);
    host= h;
    location = l;
    date = d;
    this.desc = desc;
    camera_img = c;
    matched_img = m;

  }
    /**
   * Gets the image from the camera.
   *
   * @return Can be null. Advisable to check to make sure its not null.
   */
  public byte[] getCameraImage() {

    return camera_img;
  }


  /**
   * Gets the image that matched the camera's.
   *
   * @return Can be null. Advisable to check to make sure its not null.
   */

   public byte[] getMatchedImage() {

    return matched_img;
  }



  /**
   * Get name of the host from which the event originated.
   *
   */
  public String getHost() {

    return host;
  }

  /**
   * Get the location (as specified by the event-creater) of where the
   * event happend
   *
   * @return  != null
   */
  public String getLocation() {

    return location;
  }
  /**
   * Get the time/date when this event was generated.
   *
   * @return  != null
   */
  public String getDate() {

    return date;
  }

  /**
   * Get the description of the event.
   *
   * @return != null
   */
  public String getDescription() {

    return desc;
  }

  public String getUniqueName() {

    String replacement = "[]{};':\\\",.<>/?=+-";
    String date2 = host+" "+date;//+" "+System.currentTimeMillis();
    for (int i =0; i < replacement.length();i++)
      date2 = date2.replace(replacement.charAt(i),'_');

    // Parse date to only have valid characters.
    return date2+"("+System.currentTimeMillis()+").event";
  }

  public String toString () {

    return "@"+host+" ["+location+"; "+date+"]: "+desc;
  }

}
