package apollo.common.client;

/**
 *
 * $Log: Client.java,v $
 * Revision 1.4  2002/04/14 01:25:19  uid500
 * Added comments.
 *
 * Revision 1.3  2002/04/06 21:35:18  uid500
 * Added timer countdown
 *
 * Revision 1.2  2002/04/01 02:59:46  uid500
 * Provided some setXX methods
 *
 * Revision 1.1.1.1  2002/03/30 18:51:33  uid500
 * Apollo src code import
 *
 * @author Konrad Rzeszutek (konrad@darnok.org)
 * @version $Id: Client.java,v 1.4 2002/04/14 01:25:19 uid500 Exp $
 */
import java.io.Serializable;
import java.rmi.*;
import java.net.MalformedURLException;

/**
 * Client representation.
 */
public class Client implements Comparable, Serializable{

  protected String hostname = null;
  protected String name = null;
  protected int frames = -1;
  protected int events = -1;
  protected long time_counter = 0;

  /**
   * Initialize the client.
   *
   * @param     h     The hostname of the client
   * @param     n     The name of the client
   */
  public Client(String h, String n) {
    hostname = h;
    name = n;
  }


  /**
   * Gets the hostname of the client.
   *
   * @return    String != null
   */
  public String getHost() {

    return hostname;
  }

 /**
  * Gets the name of the client.
  *
  * @return	String != null
  */
 public String getName() {

    return name;
  }
  /**
   * Compare the client alphabeticly to another client.
   *
   * @param
   *  s - the specified Client to which we will compare
   * @return
   *  a negative integer, zero, or positive integer as this client has less,
   *  equal, or greater alphabetical name
   */
  public int compareTo(Object o) {

    return name.compareTo(((Client)o).name);

  }
  public void addFrame() {
    frames++;
  }
  public void addEvent() {
    events++;
  }
  public int getEvents() {
    return events;
  }
  public int getFrames() {
    return frames;
  }
  public long getTimer() {
    return time_counter;
  }
  public void setTimer(long nr) {
    time_counter = nr;
  }
  /**
   * Gets the String representation of this object.
   *
   * @return    != null
   */
  public String toString() {

    return hostname+" Frames# "+frames+", Events# "+events;
  }
}
