package apollo.common.gui;
/**
 * $Log: MainFrame.java,v $
 * Revision 1.7  2002/04/28 19:14:17  uid500
 * Final updates.
 *
 * Revision 1.6  2002/04/25 18:59:04  uid500
 * *** empty log message ***
 *
 * Revision 1.5  2002/04/14 01:25:26  uid500
 * Added comments.
 *
 * Revision 1.4  2002/04/07 03:36:54  uid500
 * Fixed a exception problem when an JInternalFrame is being de-allocated
 *
 * Revision 1.3  2002/04/01 03:00:07  uid500
 * Rewrote some of the GUI components. Added new JInternalFrames
 *
 *
 * Copyright:    Copyright (c) 2002 by Konrad Rzeszutek
 * Company:      www.darnok.org
 *
 * @author Konrad Rzeszutek (konrad@darnok.org)
 * @version $Id: MainFrame.java,v 1.7 2002/04/28 19:14:17 uid500 Exp $
 */
import java.io.File;
import javax.swing.*;
import javax.swing.filechooser.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import apollo.common.Log;
import apollo.common.*;

/**
 * The main GUI
 *
 */
public class MainFrame extends JFrame implements FaceDetectionEventListener {

  /* Count of how many frames we have on the desktop */
  private int frames;

  /* Our GUI System.out.println logger. Public so we can all use it */
  public  LogFrame log;
  public  ClientListFrame clients;
  /* Out lists have the JMenuItems of Window| tab and JInternalFrames which
   * are accessible (protected) to the classes which are listeners for the
   * JInternalFrames - thus when user clicks on the Window| tab we can minizmie
   * or show back the frames.
   */
  protected ArrayList menuItems = new ArrayList();
  protected ArrayList windowsItems = new ArrayList();

  /* Need to open the .event files */
  private FaceDetectionEventGenerator event_handler = null;
  private JDesktopPane desktop;
  private JMenuBar jMenuBar = new JMenuBar();
  private JMenu jMenuFile = new JMenu();
  private JMenuItem jMenuFileExit = new JMenuItem();
  private JMenuItem jMenuFileOpen = new JMenuItem();


  private JMenu jMenuWindow = new JMenu();
  private JFileChooser fc = new JFileChooser();
  private WindowsActionPerformed itemActionPerformed;
  private Daemon d;   // The daemon that control's us

  /**
   * Start the Main GUI component with the right title
   * and with the hookup to the daemon (so we can call
   * it to safely terminate)
   */
  public MainFrame(String title, Daemon d) {
    super(title);

    this.d = d;

    //enableEvents(AWTEvent.WINDOW_EVENT_MASK);

    itemActionPerformed = new WindowsActionPerformed (this);

    desktop = new JDesktopPane();

    jMenuFile.setText("File");
    jMenuFileExit.setText("Exit");
    jMenuFileOpen.setText("Open");

    jMenuFileExit.addActionListener(new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        jMenuFileExit_actionPerformed(e);
      }
    });
    jMenuFileOpen.addActionListener(new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        jMenuFileOpen_actionPerformed(e);
      }
    });
    jMenuWindow.setText("Window");

    jMenuFile.add(jMenuFileOpen);
    jMenuFile.addSeparator();
    jMenuFile.add(jMenuFileExit);

    jMenuBar.add(jMenuFile);

    jMenuBar.add(jMenuWindow);

    this.setJMenuBar(jMenuBar);

    // Adds the Logging functionality
    log = new LogFrame();
    addJInternalFrame(log);

    //clients = new ClientListFrame();
    //addJInternalFrame(clients);
    this.getContentPane().add(desktop, null);

    // Speeds up
    desktop.setDragMode(JDesktopPane.OUTLINE_DRAG_MODE);
    this.setContentPane(desktop);

    // Only the .event files would show up
    fc.addChoosableFileFilter(new EventFilter());
  }


  /**
   * When the Exit item is pressed we die abrubtly.
   */
  private void jMenuFileExit_actionPerformed(ActionEvent e) {
    d.exitDaemon();
  }
  /**
   * Upon pressing the Open item we open the .event file.
   *
   * <br><b>NOTE</b>: The <it><br>registerFaceDetectionEventGenerator</it>
   * must be called so that the the event file can be read.
   */
  private void jMenuFileOpen_actionPerformed(ActionEvent e) {
            int returnVal = fc.showOpenDialog(this);

            if (returnVal == JFileChooser.APPROVE_OPTION) {
                if (event_handler != null)
                  try {
                    // Read the file
                    event_handler.readEvent(fc.getSelectedFile().getAbsolutePath());
                  } catch (Exception error) {
                    log.normalError(error.getMessage());
                  }
              }
  }

  /**Overridden so we can exit when window is closed*/
  protected void processWindowEvent(WindowEvent e) {
    super.processWindowEvent(e);
    if (e.getID() == WindowEvent.WINDOW_CLOSING) {
      jMenuFileExit_actionPerformed(null);
    }
  }

  /**
   * Registers an <code>FaceDetectionEventGenerator</code> with the GUI. This is
   * neccessary to open the .event files.
   *
   * @param event An implementation of the FaceDetectionEventGenerator interface.
   */
  public void registerFaceDetectionEventGenerator(FaceDetectionEventGenerator event) {

    event_handler = event;
  }
  /**
   * Adds to the desktop and to the Window| tab an JInternalFrame
   *
   * @param frame != null
   */
  public synchronized void addJInternalFrame(JInternalFrame frame) {

    JMenuItem item = new JMenuItem(frame.getTitle());

    frame.addInternalFrameListener(itemActionPerformed);
    item.addActionListener(itemActionPerformed);

    windowsItems.add(frame);
    menuItems.add(item);

    // Do the magic - add it to the jMenuWindow
    jMenuWindow.add(item);

    desktop.add(frame);

    frame.setLocation(this.getX()+20*frames, this.getY() + 20*frames);
    frame.setVisible(true);
    desktop.setSelectedFrame(frame);
    frames++;
  }

  /**
   * Unregisters an JInternalWindow with the desktop and removes itself
   * from the Window| tab.
   *
   * @param f Must be already registered.
   */
  public synchronized void removeJInternalFrame(JInternalFrame f) {

        JInternalFrame frame = null;

        int j =0;

          Iterator i = windowsItems.iterator();

          while (i.hasNext() ) {
            frame = (JInternalFrame)i.next();
            if (frame == f) break;
            j++;
          }
        try {
        synchronized (menuItems) {
          jMenuWindow.remove((JMenuItem)menuItems.get(j));

          menuItems.remove(j);
        }
        synchronized (windowsItems) {
          windowsItems.remove(j);
        }
        } catch (Exception e) { }
        frames--;
  }
  /**
   * Receiver for the event.
   *
   * @param event Is guaranteed not to be null.
   */
  public void faceDetectionEventReceived(FaceDetectionEvent event) {

    FaceDetectionWindow face = new FaceDetectionWindow(event);
    addJInternalFrame(face);

  }


  /**
   * Class taking care of the ActionPerformed and JInternalFrame action's
   *
   *
   */
  public class WindowsActionPerformed implements ActionListener,
    InternalFrameListener {

    private MainFrame the_frame;

    /**
     * Initialize this object.
     *
     * @param f The top frame
     */
    public WindowsActionPerformed(MainFrame f) {

      the_frame = f;
    }
    public void internalFrameClosing(InternalFrameEvent e) { }
    public void internalFrameClosed(InternalFrameEvent e) {

      the_frame.removeJInternalFrame(e.getInternalFrame());

    }

    public void internalFrameOpened(InternalFrameEvent e) {   }

    public void internalFrameIconified(InternalFrameEvent e) {    }

    public void internalFrameDeiconified(InternalFrameEvent e) {   }

    public void internalFrameActivated(InternalFrameEvent e) {   }

    public void internalFrameDeactivated(InternalFrameEvent e) {   }

    /**
     * When an user clicks on Window|<xx> JMenuItem this method is
     * invoked. Finds the JInternalFrame from the Window tab and if
     * its iconfied, it becomes de-iconfied. If its minimized it becomes
     * restored. If its restored and selected, it becomes iconfied.
     *
     * @param e != null
     */
    public void actionPerformed(ActionEvent e) {

          JInternalFrame frame = null;
          JMenuItem item = null;
          int j =0;
          Iterator i = the_frame.menuItems.iterator();

          while (i.hasNext() ) {
            item = (JMenuItem)i.next();
            if (item == e.getSource()) break;
            j++;
          }

          frame = (JInternalFrame)the_frame.windowsItems.get(j);

        try {
          if (frame.isClosed()) {
            //System.out.println("(face.isClosed())");
          }
          else if (!frame.isSelected()) {
            //System.out.println("!isSelected");
            frame.setSelected(true);
            if (frame.isIcon()) {
              frame.setIcon(false);
              frame.setEnabled(true);
            }
          }
          else if (frame.isIcon()) {
            //System.out.println("(face.isIcon())");
             frame.setIcon(false);
             frame.setVisible(true);
          }
          else if (!frame.isIcon()) {
            //System.out.println("!face.isIcon()");
            frame.setIcon(true);
            frame.setEnabled(false);
          }
        } catch (Exception error) { }
      }
    }

  /**
   * Simple class responsible for filtering extensions for the FileChooser.
   */
  public class EventFilter extends FileFilter {

    /**
     * Accept only directories and .event files.
     */
    public boolean accept(File f) {

        if (f.isDirectory()) {
            return true;
        }

        String extension = f.getName();
        int i = extension.lastIndexOf('.');

        if (i > 0 &&  i < extension.length() - 1) {
            extension = extension.substring(i+1).toLowerCase();
        }

        if (extension != null) {
            if (extension.equals("event")){
                    return true;
            } else {
                return false;
            }
        }

        return false;
    }

    // The description of this filter
    public String getDescription() {
        return "*.events";
    }
  }
}
