package apollo.common.jini;
/**
 * @author: Konrad Rzeszutek <konrad@www.darnok.org>
 * @description:
 *
 * $Log: JiniServerRegister.java,v $
 * Revision 1.7  2002/04/28 19:14:18  uid500
 * Final updates.
 *
 * Revision 1.6  2002/04/14 01:25:28  uid500
 * Added comments.
 *
 * Revision 1.5  2002/04/06 21:35:41  uid500
 * Removed RMISecurityManager -conflicts with JMF
 *
 * Revision 1.4  2002/04/02 02:11:26  uid500
 * Fixed minor bugs
 *
 * Revision 1.3  2002/04/01 03:00:25  uid500
 * Made the Jini service registration easier
 *
 * Revision 1.2  2002/03/31 03:22:55  uid500
 * *** empty log message ***
 *
 * Revision 1.1.1.1  2002/03/30 18:51:33  uid500
 * Apollo src code import
 *
 * @version: $Id: JiniServerRegister.java,v 1.7 2002/04/28 19:14:18 uid500 Exp $
 */

import java.rmi.*;
import net.jini.core.discovery.*;
import net.jini.discovery.*;
import net.jini.core.lookup.*;
import net.jini.core.lease.*;
import net.jini.lease.*;
import java.io.*;
import apollo.common.Log;
import java.net.*;
import java.rmi.server.*;

public class JiniServerRegister implements DiscoveryListener, LeaseListener{

    //Variable Declaration
    protected LeaseRenewalManager leaseManager;
    private ServiceItem s;
    private Log log;

    /**
     *  Instantiates the registration  process and creates the service
     *
     * @param s The <code>JiniServerCompenet</code> instantiated for this
     *  Jini service.
     */
    public JiniServerRegister(ServiceItem s, Log log ){

        this.s = s;
        this.log = log;
 	//System.setSecurityManager(new RMISecurityManager( ));
	leaseManager = new LeaseRenewalManager();
	LookupDiscovery discover = null;

	try{
	    discover = new LookupDiscovery(LookupDiscovery.ALL_GROUPS);
	}catch(Exception e){
          log.println(e.getMessage(), Log.ERROR);
	}
	discover.addDiscoveryListener(this);
    }

    public synchronized void discovered(DiscoveryEvent dev){
	//get the lookup servers currently available

	ServiceRegistrar[] registrars = dev.getRegistrars( );

	try{
	    for(int i=0; i<registrars.length; i++){

		ServiceRegistrar registrar = registrars[i];
		log.println("A new registrar found at "+
                  registrar.getLocator( ).getHost( ), "Jini");

		ServiceItem item = s;
		// lease the service for 5 minutes but the
		//actual lease time is the lookupserver's call

                //log.println(s.service.toString(),"Jini");
		ServiceRegistration reg = null;
		long actuallease = 0;
		try{
		    reg = registrar.register(item, 3000L);
		    actuallease = (reg.getLease().getExpiration() -
				   System.currentTimeMillis()) / 1000;
		}catch(RemoteException e){
		    log.println(e.getMessage(), Log.ERROR);
		}

		log.println("Service registered, expires in " +
				   actuallease + " seconds. Leasing forever.",
                                    "Jini");
		leaseManager.renewUntil(reg.getLease( ), Lease.FOREVER,this);
	    }//end for
	}catch(Exception e){
	    log.println(e.getMessage(),Log.ERROR);
	}
    }//end method

    public synchronized void discarded(DiscoveryEvent dev){

      //System.out.println("Discarded "+dev);
    }

    public void notify(LeaseRenewalEvent e){
	//System.out.println("Lease Expired "+e.toString( ));

    }

}
