package apollo.common.save;

/**
 * Title:        Apollo - Dynamic Distributed Face Recognition Security System
 * @author Konrad Rzeszutek (konrad@darnok.org)
 * @version $Id: Saver.java,v 1.4 2002/04/16 19:22:15 uid500 Exp $
 */

import apollo.common.client.Client;
import apollo.common.events.*;
import java.net.*;
import javax.media.*;
import javax.media.format.*;
import javax.media.protocol.*;
import java.io.*;
public interface Saver {

    /**
     * Open the connection specified by the <code>inputURL</code>.
     *
     * @throws MalformedURLException if the <code>inputURL</code> is invalid.
     * @throws NoProcessorException is thrown when Manager can't find a Processor for a particular URL, MediaLocator or ProcessorModel
     * @throws NoDataSourceException is thrown when a DataSource can't be found for a particular URL or MediaLocator
     * @throws UnknownProblemException if something terribly wrong happend
     * @throws IOException if there are problems reading the data
     * @throws IncompatibleSourceException if the input format is a wrong one and not the one that's expected.
     * @throws UnsupportedFormatException if the format of the input media does not match the one that's required.
     */
    public void open(MediaLocator url) throws MalformedURLException,
      NoProcessorException, NoDataSourceException,
      UnknownProblemException,  IOException, UnsupportedPlugInException,
      IncompatibleSourceException, UnsupportedFormatException;


    /**
     * Open the connection specified by the <code>ds</code> DataSource.
     *
     * @throws MalformedURLException if the <code>inputURL</code> is invalid.
     * @throws NoProcessorException is thrown when Manager can't find a Processor for a particular URL, MediaLocator or ProcessorModel
     * @throws NoDataSourceException is thrown when a DataSource can't be found for a particular URL or MediaLocator
     * @throws UnknownProblemException if something terribly wrong happend
     * @throws IOException if there are problems reading the data
     * @throws IncompatibleSourceException if the input format is a wrong one and not the one that's expected.
     * @throws UnsupportedFormatException if the format of the input media does not match the one that's required.
     *
     */
   public void open(DataSource url) throws MalformedURLException,
      NoProcessorException, NoDataSourceException,
      UnknownProblemException,  IOException,UnsupportedPlugInException,
      IncompatibleSourceException, UnsupportedFormatException;

    /**
     * Flush the connection (if its openned).
     */
    public void flush();
     /**
     * Close the connection (if its openned).
     *
     */
    public void close();


    /**
     * Register the client with this object.
     *
     */
   public void registerClient(Client c);


}