package apollo.demeter;

/**
 * $Log: DemeterDaemon.java,v $
 * Revision 1.7  2002/04/28 19:14:23  uid500
 * Final updates.
 *
 * Revision 1.6  2002/04/25 18:59:09  uid500
 * *** empty log message ***
 *
 * Revision 1.5  2002/04/16 19:22:32  uid500
 * Made it work with Hermes client count
 *
 * Revision 1.4  2002/04/14 01:25:41  uid500
 * Added comments.
 *
 * Revision 1.3  2002/04/07 03:37:27  uid500
 * Fixed the RTP start/close logic.
 *
 * Revision 1.2  2002/04/06 21:36:30  uid500
 * Fixed SecuritManger exception
 *
 * Revision 1.1  2002/04/02 02:11:38  uid500
 * Created the demeter daemons
 *
 * Copyright:    Copyright (c) 2002
 * Company:      www.darnok.org
 * @author Konrad Rzeszutek (konrad@darnok.org)
 * @version $Id: DemeterDaemon.java,v 1.7 2002/04/28 19:14:23 uid500 Exp $
 */
import apollo.common.*;
import apollo.common.gui.*;
import apollo.common.tui.*;
import apollo.common.jini.*;
import apollo.common.client.*;
import apollo.hermes.rmi.*;
import apollo.common.movie.*;
import apollo.hermes.servers.*;
import apollo.demeter.rmi.*;
import apollo.demeter.jini.*;
import apollo.demeter.save.*;
import apollo.common.save.*;
import apollo.demeter.client.*;
import java.rmi.*;
import java.rmi.server.*;
import java.net.*;
import java.io.*;
import net.jini.core.lookup.*;
import javax.media.*;

/**
 * Demeter was the Goddess of crops, agriculture and fertility.
 *
 * And so does this daemon - deals with crops and fertility. Crops of images that is.
 * And it generates (read: grows) movies.
 *
 * All of the images are received by RTP and are stored in DEMETERPATH variable (or
 * c:/apollo/demeter/[images|movies] by default)
 */
public class DemeterDaemon implements Daemon{

  private final static String DAEMON_NAME  = "DemeterD";
  private final static String DAEMON_VER   = "$Id: DemeterDaemon.java,v 1.7 2002/04/28 19:14:23 uid500 Exp $";

  /**
   * Max amounts of clietns we can handle.
   */
  public static int DAEMON_MAX_CLIENTS = 32;

  private DemeterRMIServer demeter_rmi;
  private JiniServerRegister demeter_register;
  private ServerRMIInfo hermes_rmi;
  private JiniDemeterInfo demeter_jini;
  private Log log = new TextLog();
  private MainFrame frame;
  private String hermes_url;
  private String demeter_url;
  private String ver;
  private ClientList clients;
  private String RMI_server;
  private int RMI_port;
  private Liaison liaison;
  private MovieCreator movie;
  /**
   * Start the MorsDaemon.
   *
   * @param nogui If the system should run without a gui
   * @param dir Directory where images and movies need to be stored.
   */
  public DemeterDaemon(boolean nogui, String dir) {


    // Find out about ourselves - what's our IP, where the RMI server is,
    // on what port it resides
    getHostInfo();

    hermes_url = "rmi://"+RMI_server+":"+RMI_port+"/"+
      JiniServerComponentInfo.DEMETER;
    demeter_url = "rmi://"+RMI_server+":"+RMI_port+"/"+
      DemeterRMIInterface.DEMETER_RMI;


    if (!nogui) {
      frame = new MainFrame(DAEMON_NAME, this);
      frame.setSize(800,600);
      frame.validate();
      frame.setVisible(true);

      Object waitFrame = new Object();
      while (frame.log == null) {
        synchronized (waitFrame) {
          try {
            waitFrame.wait(1000);
          } catch (InterruptedException e) { }
        }
      }
      log = ((Log)frame.log);
    }

    // Start the Hermes RMI server
    //  System.setSecurityManager(new RMISecurityManager()); /* This breaks JMF */

    try {
      hermes_rmi = new ServerRMIInfo(DAEMON_MAX_CLIENTS,demeter_url,ver);
      demeter_rmi = new DemeterRMIServer();
    } catch (RemoteException e) { //e.printStackTrace();
      log.normalError(e.getMessage());
    }


    // Bind RMI
    try {
          log.println("Rebinding "+hermes_url,Log.INFO);
          Naming.rebind(hermes_url,hermes_rmi);
          log.println("Rebinding "+demeter_url,Log.INFO);
          Naming.rebind(demeter_url,demeter_rmi);
    } catch (Exception  e) {
      if (log.fatalError(e.getMessage())) exitDaemon();
      // Couldn't bind to RMI. Fatal error.
    }

    log.println("Started "+hermes_url+" server ..",Log.INFO);
    log.println("Started "+demeter_url+" server ..",Log.INFO);

    // Start the Jini server

    demeter_jini = new JiniDemeterInfo(RMI_server, RMI_port);
    demeter_register = new JiniServerRegister(
      new ServiceItem(null,demeter_jini,null),log);
    log.println("Started Jini server ..",Log.INFO);



    // Link events to prospective listeners.
    if (frame != null) {
      frame.clients = new ClientListFrame();
      frame.addJInternalFrame(frame.clients);
    }
    clients = new ClientList();
    movie = new MovieCreator(log);
    liaison = new Liaison(clients,log,frame,movie, dir, hermes_rmi);

    // Link Liaison
    demeter_rmi.registerLiaison(liaison);

    log.println("Daemon started. ",Log.INFO);

}
  /**
   * Really bad stuff happens here. We close everything we can
   * cleanly and then exit.
   */
  public void exitDaemon() {

    if (movie != null) {
      movie.killThread();
      movie.closeAll(); 
    }
    movie = null;
    liaison = null;
  // How about terminating the existing threads?
    try {
      Naming.unbind(demeter_url);
      Naming.unbind(hermes_url);
    } catch (Exception e) { } // Ignore them
    // Kill the GUI
    frame = null;

    System.exit(0);
  }

   /**
   * Find out what our IP is. Where the RMI server resides, and what port
   * is the RMI server up on. These info are assumed to be the local machine,
   * but if the RMI server is located somewhere else, the
   *  RMISERVER variable and RMIPORT hold the values.
   */
  public void getHostInfo() {

      try {
        RMI_server = System.getProperty("RMISERVER",
          java.net.InetAddress.getLocalHost().getHostAddress());

      } catch (Exception e) {
        if (log.fatalError(e.getMessage())) exitDaemon();
      }
      String tmp = System.getProperty("RMIPORT","1098");
      RMI_port = 1098;
      try {
        RMI_port = Integer.parseInt(tmp);
      } catch (Exception e) { }

  }
  private static void prUsage() {

        System.out.println("java -jar demeter [-nogui] -d directory");
        System.exit(-2);

  }
  public static void main(String[] args) {

    int i = 0;
    boolean nogui = false;
    String dir = null;
    while (i < args.length) {
      if (args[i].equals("-nogui")) {
		i++;
                nogui = true;
      } else if (args[i].equals("-h")) {
        prUsage();
      } else if (args[i].equals("-d")) {
          i++;
          if (i >= args.length)
            prUsage();
          dir = args[i];
      }
      i++;
    }

    if (dir == null)
      prUsage();
    new DemeterDaemon(nogui, dir);

  }
}
