package apollo.demeter.client;

/**
 * $Log: DemeterClient.java,v $
 * Revision 1.3  2002/04/14 01:25:43  uid500
 * Added comments.
 *
 * Revision 1.2  2002/04/07 03:37:41  uid500
 * added registerSaver(Saver)
 *
 * Revision 1.1  2002/04/06 21:36:49  uid500
 * NEw Client - with DemeterFrameWindow support
 *
 * Copyright:    Copyright (c) 2002
 * Company:      www.darnok.org
 * @author Konrad Rzeszutek (konrad@darnok.org)
 * @version $Id: DemeterClient.java,v 1.3 2002/04/14 01:25:43 uid500 Exp $
 */
import apollo.common.client.*;
import apollo.common.save.Saver;
import apollo.demeter.gui.*;
//import apollo.common.ApolloImage;
import javax.swing.JInternalFrame;

/**
 * An extension to the Client.
 *
 * This object provides additional functionality: registering a Saver component,
 * creating a GUI, receiving the image and movie directory name (where
 * respectivly images and movies are saved).
 *
 */
public class DemeterClient extends Client{

  private String base_dir;
  private Saver save;
  private DemeterFrame frame;

  /**
   * Creates the DemeterClient
   *
   * @param h The hostname of the machiene.
   * @param n The name of hte machine.
   * @param base_dir  The base_directory where images/movies will saved / generated.
   */
  public DemeterClient(String h, String n, String base_dir) {

    super(h,n);
    this.base_dir = base_dir;
    frame = new DemeterFrame(this);
  }

  /**
   * Register the Saver which updates the image to the GUI
   *
   * @param s != null
   */
  public void registerSaver(Saver s) {

    this.save = s;
  }

  /**
   * Gets the full pathname to the directory where images
   * are stored.
   *
   * @return != null
   */
  public String getImageDir() {

    return base_dir + "/images/"+super.getHost();
  }

  /**
   * Returns the full pathname to the directory where movies
   * are generated.
   *
   * @return != null
   *
   */
  public String getMovieDir() {

   return base_dir + "/movies/"+super.getHost();
  }

  /**
   * Gets the GUI component used to display images/frame count/
   * hostname/location information
   *
   * @return  != null
   */
  public JInternalFrame getFrame() {

    return frame;
  }

  /**
   * Sets the image on the screen.
   */
  public void setImage(byte[] img) {

    if (frame != null)
      frame.setImage(img);
  }

  /**
   * Disposes off everything.
   */
  public void close() {

    frame.dispose();
    frame = null;
    save = null;
    System.gc();
  }
  /**
   * Gets the Saver handling this client
   *
   * @return <code>null</code> if registerSaver() hasn't been called.
   */
  public Saver getSaver() {

    return save;
  }
}