package apollo.demeter.rmi;

/**
 * $Log: DemeterRMIServer.java,v $
 * Revision 1.4  2002/04/16 19:23:05  uid500
 * Added connectionOk()
 *
 * Revision 1.3  2002/04/14 01:25:48  uid500
 * Added comments.
 *
 * Revision 1.2  2002/04/07 03:37:49  uid500
 * Added Exceptions
 *
 * Revision 1.1  2002/04/06 21:37:21  uid500
 * RMI server side for Demeter
 *
 * @author Konrad Rzeszutek (konrad@darnok.org)
 * @version $Id: DemeterRMIServer.java,v 1.4 2002/04/16 19:23:05 uid500 Exp $
 */
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.net.*;
import java.rmi.server.UnicastRemoteObject;
import apollo.demeter.*;
import apollo.common.events.*;
import javax.media.*;

/**
 * Demeter's RMI services implemented. The <code>registerLiaison</code>
 * must be called in order for these services to work properly.
 *
 */
public class DemeterRMIServer extends UnicastRemoteObject
                          implements DemeterRMIInterface {

  private Liaison l;
  public DemeterRMIServer() throws RemoteException {  }

  /**
   * Negotiates an RTP connection to the server
   *
   * @param url The URL for the RTP stream - P-to-P connection
   */
  public void openRTP(MediaLocator url) throws RemoteException {

    if (l != null) {
      l.open(url);
    } else
      System.err.println("Implementation error");

  }
  /**
    * Checks the connection. Since the RTP connection is UDP, we don't know
    * if the other side still has the connection active.
    */
  public void isConnectionOk() throws RemoteException {

  }
   /**
    * Notifies the server that the client wishes to terminate the
    * RTP connection.
    *
    * @param url  The clients URL used in establishing the connection
    */
  public void closeRTP(MediaLocator url) throws RemoteException,
    IsNotRegisteredException,MalformedURLException {

    if (l != null) {
      l.close(url);
    } else
      System.err.println("Implementation error");

  }
  /**
   * Register the RMI portal with the liaison object. This is neccesary to
   * have the openRTP, closeRTP work properly.
   */
  public void registerLiaison(Liaison l) {
    this.l = l;
  }
}