package apollo.demeter.save;

/**
 * @version $Id: RTPtoJPG.java,v 1.3 2002/04/28 19:14:27 uid500 Exp $
 */


import java.io.IOException;
import java.util.Vector;
import java.util.Enumeration;
import javax.media.*;
import javax.media.format.*;
import javax.media.protocol.*;
import javax.media.protocol.DataSource;
import javax.media.datasink.*;
import javax.media.control.MonitorControl;
import javax.media.control.TrackControl;
import java.io.*;
import apollo.common.*;
import apollo.common.client.*;
import apollo.common.events.*;
import apollo.common.save.*;
import apollo.demeter.client.DemeterClient;
import java.net.*;
import apollo.demeter.client.*;

/**
 * RTP to JPG processor.
 *
 * Receives RTP input from the given location (specified in open()) and
 * saves incoming data in the directory specified by the DemeterClient
 * (DemeterClient.getImageDir()) based on the count of frames.
 *
 *
 */
public class RTPtoJPG implements  Saver, ControllerListener, DataSinkListener {

    private JPGDataSourceHandler handler;
    private Processor p;

    private Object waitSync = new Object();

    private boolean stateTransitionOK = true;
    static boolean monitorOn = false;
    private DemeterClient c;
    private Log log;
    private Codec[] codec;
    /**
     * Initializes the object.
     *
     * @param log For displaying useful information to the user
     * @param c Client using this object.
     */
    public RTPtoJPG(Log log, DemeterClient c, Codec[] codec) {
      this.log = log;
      this.c = c;
      this.codec= codec;
    }

    /**
     * Register the client with this object.
     *
     */
    public void registerClient(Client client) {

      this.c = (DemeterClient)client;

    }

    /**
     * Close the connection (if its openned).
     *
     */
    public void close() {
      kill();
    }

    /**
     * Flush the connection (if its openned).
     */
    public void flush() {}

    /**
     * Open the connection specified by the <code>inputURL</code>.
     * <br><b>NOTE</b>: An RTP connection can take a long time to open,
     * especially if the packets arrive seldom.
     *
     * @throws MalformedURLException if the <code>inputURL</code> is invalid.
     * @throws NoProcessorException is thrown when Manager can't find a Processor for a particular URL, MediaLocator or ProcessorModel
     * @throws NoDataSourceException is thrown when a DataSource can't be found for a particular URL or MediaLocator
     * @throws UnknownProblemException if something terribly wrong happend
     * @throws IOException if there are problems reading the data
     * @throws IncompatibleSourceException if the input format is a wrong one and not the one that's expected.
     * @throws UnsupportedFormatException if the format of the input media does not match the one that's required.
     */
    public void open (MediaLocator inputURL) throws MalformedURLException,
      NoProcessorException, NoDataSourceException,
      UnknownProblemException,  IOException, UnsupportedPlugInException,
      IncompatibleSourceException, UnsupportedFormatException {


	DataSource ds = Manager.createDataSource(inputURL);

        open(ds);

    }

    /**
     * Open the connection specified by the <code>ds</code> DataSource.
     * <br><b>NOTE</b>: An RTP connection can take a long time to open,
     * especially if the packets arrive seldom.
     *
     * @throws MalformedURLException if the <code>inputURL</code> is invalid.
     * @throws NoProcessorException is thrown when Manager can't find a Processor for a particular URL, MediaLocator or ProcessorModel
     * @throws NoDataSourceException is thrown when a DataSource can't be found for a particular URL or MediaLocator
     * @throws UnknownProblemException if something terribly wrong happend
     * @throws IOException if there are problems reading the data
     * @throws IncompatibleSourceException if the input format is a wrong one and not the one that's expected.
     * @throws UnsupportedFormatException if the format of the input media does not match the one that's required.
     *
     */
    public void open(DataSource ds) throws NoProcessorException,
      UnknownProblemException,  IOException,
      IncompatibleSourceException, UnsupportedFormatException,
      UnsupportedPlugInException {


        p = Manager.createProcessor(ds);
	p.addControllerListener(this);

	// Put the Processor into configured state.
	p.configure();

	if (!waitForState(p.Configured))
	    throw new UnknownProblemException("Failed to configure the processor.");

	// Get the raw output from the processor.
	p.setContentDescriptor(new ContentDescriptor(ContentDescriptor.MIXED));


	TrackControl tc[] = p.getTrackControls();
        TrackControl track =null;
	if (tc == null)
            throw new UnsupportedFormatException("The input  does not support"+
              " track control!",null);

	// Search for the track control for the video track.

	for (int i = 0; i < tc.length; i++) {
	    if (tc[i].getFormat() instanceof VideoFormat) {
		//videoTrack = tc[i];
                tc[i].setFormat(new VideoFormat(VideoFormat.JPEG));
                track = tc[i];
		break;
	    }
	}
        if (codec != null) {
          track.setCodecChain(codec);
        }
	p.realize();
	if (!waitForState(p.Realized))
            throw new UnknownProblemException("Failed to realize the processor.");

	// Get the output DataSource from the processor and
	// hook it up to the DataSourceHandler.
	DataSource ods = p.getDataOutput();

	handler = new JPGDataSourceHandler(log,c);
        handler.setSource(ods);
	handler.addDataSinkListener(this);
	handler.start();

	// Prefetch the processor.
	p.prefetch();

	if (!waitForState(p.Prefetched))
	  throw new UnknownProblemException("Unable to prefetch the processor.");

	// Start the processor.
	p.start();

        log.println("Started the processor ..",c.getHost());

    }
    /**
     * Kills (closes the connection) for the RTP stream in safe way.
     */
    public void kill() {

      synchronized (waitSync) {
	    stateTransitionOK = false;
            waitSync.notifyAll();
      }
      p.stop();
      p.close();
      handler.stop();
      handler.close();
      handler = null;
      p = null;
      // Call the garbage collector
      System.gc();
    }


    /**
     * Block until the processor has transitioned to the given state.
     * Return false if the transition failed.
     */
    private boolean waitForState(int state) {
	synchronized (waitSync) {
	    try {
		while (p.getState() < state && stateTransitionOK)
		    waitSync.wait();
	    } catch (Exception e) {}
	}
	return stateTransitionOK;
    }


    /**
     * Controller Listener.
     */
    public void controllerUpdate(ControllerEvent evt) {

	if (evt instanceof ConfigureCompleteEvent ||
	    evt instanceof RealizeCompleteEvent ||
	    evt instanceof PrefetchCompleteEvent) {
	    synchronized (waitSync) {
		stateTransitionOK = true;
		waitSync.notifyAll();
	    }
	} else if (evt instanceof ResourceUnavailableEvent) {
	    synchronized (waitSync) {
		stateTransitionOK = false;
		waitSync.notifyAll();
	    }
	} else if (evt instanceof EndOfMediaEvent) {
	    p.close();
	} else if (evt instanceof SizeChangeEvent) {
	}
    }


    /**
     * DataSink Listener
     */
    public void dataSinkUpdate(DataSinkEvent evt) {

	if (evt instanceof EndOfStreamEvent) {
	    //System.err.println("All done!");
	    evt.getSourceDataSink().close();
	    //System.exit(0);
	}
    }



    class JPGDataSourceHandler implements DataSink, BufferTransferHandler {
	private DataSource source;
	private PullBufferStream pullStrms[] = null;
	private PushBufferStream pushStrms[] = null;

	// Data sink listeners.
	private Vector listeners = new Vector(1);

	// Stored all the streams that are not yet finished (i.e. EOM
	// has not been received.
	private SourceStream unfinishedStrms[] = null;
	private int counter = 0;
	// Loop threads to pull data from a PullBufferDataSource.
	// There is one thread per each PullSourceStream.
	private Loop loops[] = null;
        private String image_dir;
        private Log log;
	private Buffer readBuffer;
	private RandomAccessFile raFile = null;

        private DemeterClient c;
        private long time;

        public JPGDataSourceHandler(Log log, DemeterClient c) {


          this.log  = log;
          this.c = c;
          time = System.currentTimeMillis();//new java.util.Date().getTime();


        }
	/**
	 * Sets the media source this <code>MediaHandler</code>
	 * should use to obtain content.
	 */
	public void setSource(DataSource source) throws
          IncompatibleSourceException {

	    // Different types of DataSources need to handled differently.
	    if (source instanceof PushBufferDataSource) {

		pushStrms = ((PushBufferDataSource)source).getStreams();
		unfinishedStrms = new SourceStream[pushStrms.length];

		// Set the transfer handler to receive pushed data from
		// the push DataSource.
		for (int i = 0; i < pushStrms.length; i++) {
		    pushStrms[i].setTransferHandler(this);
		    unfinishedStrms[i] = pushStrms[i];
		}


	    } else if (source instanceof PullBufferDataSource) {

		pullStrms = ((PullBufferDataSource)source).getStreams();
		unfinishedStrms = new SourceStream[pullStrms.length];

		// For pull data sources, we'll start a thread per
		// stream to pull data from the source.
		loops = new Loop[pullStrms.length];
		for (int i = 0; i < pullStrms.length; i++) {
		    loops[i] = new Loop(this, pullStrms[i]);
		    unfinishedStrms[i] = pullStrms[i];
		}

	    } else {

		// This handler only handles push or pull buffer datasource.
		throw new IncompatibleSourceException();

	    }

	    this.source = source;
	    readBuffer = new Buffer();

	}


	/**
	 * For completeness, DataSink's require this method.
	 * But we don't need it.
	 */
	public void setOutputLocator(MediaLocator ml) {

		// This is where we would actually output data to a file..QuickTime or MPEG
	}


	public MediaLocator getOutputLocator() {
	    return null;
	}


	public String getContentType() {
	    return source.getContentType();
	}


	/**
	 * Our DataSink does not need to be opened.
	 */
	public void open() {
	}


	public void start() {
	    try {
		source.start();
	    } catch (IOException e) {
		log.println(e.getMessage(),c.getHost());
	    }

	    // Start the processing loop if we are dealing with a
	    // PullBufferDataSource.
	    if (loops != null) {
		for (int i = 0; i < loops.length; i++)
		    loops[i].restart();
	    }
            log.println("Starting to write. ", c.getHost());
	}


	public void stop() {
	    try {
		source.stop();
	    } catch (IOException e) {
		log.println(e.getMessage(), c.getHost());
	    }

	    // Start the processing loop if we are dealing with a
	    // PullBufferDataSource.
	    if (loops != null) {
		for (int i = 0; i < loops.length; i++)
		    loops[i].pause();
	    }
            log.println("Stopping write process. ", c.getHost());
	}


	public void close() {
	    stop();
	    if (loops != null) {
		for (int i = 0; i < loops.length; i++)
		    loops[i].kill();
	    }
	}


	public void addDataSinkListener(DataSinkListener dsl) {
	    if (dsl != null)
		if (!listeners.contains(dsl))
		    listeners.addElement(dsl);
	}


	public void removeDataSinkListener(DataSinkListener dsl) {
	    if (dsl != null)
		listeners.removeElement(dsl);
	}


	protected void sendEvent(DataSinkEvent event) {
	    if (!listeners.isEmpty()) {
		synchronized (listeners) {
		    Enumeration list = listeners.elements();
		    while (list.hasMoreElements()) {
			DataSinkListener listener =
				(DataSinkListener)list.nextElement();
			listener.dataSinkUpdate(event);
		    }
		}
	    }
	}


	/**
	 * This will get called when there's data pushed from the
	 * PushBufferDataSource.
	 */
	public void transferData(PushBufferStream stream) {

	    try {
		stream.read(readBuffer);
	    } catch (IOException e) {
		System.err.println(e);
		sendEvent(new DataSinkErrorEvent(this, e.getMessage()));
		return;
	    }

	    saveBuffer(readBuffer);

	    // Check to see if we are done with all the streams.
	    if (readBuffer.isEOM() && checkDone(stream)) {
		sendEvent(new EndOfStreamEvent(this));
	    }
	}


	/**
	 * This is called from the Loop thread to pull data from
	 * the PullBufferStream.
	 */
	public boolean readPullData(PullBufferStream stream) {
	    try {
		stream.read(readBuffer);
	    } catch (IOException e) {
		System.err.println(e);
		return true;
	    }


	    saveBuffer(readBuffer);

	    if (readBuffer.isEOM()) {
	        // Check to see if we are done with all the streams.
		if (checkDone(stream)) {
		    //System.err.println("All done!");
		    close();
		}
		return true;
	    }
	    return false;
	}


	/**
	 * Check to see if all the streams are processed.
	 */
	public boolean checkDone(SourceStream strm) {
	    boolean done = true;

	    for (int i = 0; i < unfinishedStrms.length; i++) {
		if (strm == unfinishedStrms[i])
		    unfinishedStrms[i] = null;
		else if (unfinishedStrms[i] != null) {
		    // There's at least one stream that's not done.
		    done = false;
		}
	    }
	    return done;
	}


	void saveBuffer(Buffer buffer) {

          /*
	    if (buffer.getFormat() instanceof AudioFormat)
		System.err.println("Read audio data:");
	    else
		System.err.println("Read video data:");

	    System.err.println("  Time stamp: " + buffer.getTimeStamp());
	    System.err.println("  Sequence #: " + buffer.getSequenceNumber());
	    System.err.println("  Data length: " + buffer.getLength());
            */
	    if (!buffer.isEOM()) {
              //log.println("Data length: " + buffer.getLength(),Log.INFO);
	      String name = c.getImageDir();
              String c_str = Integer.toString(counter);
              int j = c_str.length();

              for (int i=8; i > j;i--)
                c_str = "0" + c_str;

              name += "/" + c_str + ".jpg";

              counter ++;
              byte data[] = null;
	      try {
  	        raFile = new RandomAccessFile(name, "rw");
	        data = new byte[buffer.getLength()];

	        if (buffer.getData() instanceof byte[])
		    System.arraycopy(buffer.getData(),0,data,0,buffer.getLength());

	        raFile.write(data);
	        raFile.close();
	      } catch (Exception e) {
                log.println(e.getMessage(), Log.ERROR);
              }
              // Next part, update the GUI, every 10 seconds
              if (c != null) {
                // Updates the count.
                c.addFrame();
                long now_time = System.currentTimeMillis();//new java.util.Date().getTime();

                if ((time + 10000) < now_time) {
                  time = now_time;
                  // Puts a new pic in the GUI
                  c.setImage(data);
                  //c.setImage(new JPEGImage(Long.toString(buffer.getTimeStamp()), data));
                }
              }
            }
	}

	public Object [] getControls() {
	    return new Object[0];
	}

	public Object getControl(String name) {
	    return null;
	}
    }

    class TakeawayLoop extends Thread {

      private DataSource ds;
      private RTPtoJPG r;

      public TakeawayLoop(RTPtoJPG r, DataSource ds) {

        this.ds = ds;
        this.r =r;
      }
      public void run() {

     //   r.open(ds);
      }
    }
    /**
     * A thread class to implement a processing loop.
     * This loop reads data from a PullBufferDataSource.
     */
    class Loop extends Thread {

	JPGDataSourceHandler handler;
	PullBufferStream stream;
	boolean paused = true;
	boolean killed = false;

	public Loop(JPGDataSourceHandler handler, PullBufferStream stream) {
	    this.handler = handler;
	    this.stream = stream;
	    start();
	}

	public synchronized void restart() {
	    paused = false;
	    notify();
	}

	/**
	 * This is the correct way to pause a thread; unlike suspend.
	 */
	public synchronized void pause() {
	    paused = true;
	}

	/**
	 * This is the correct way to kill a thread; unlike stop.
	 */
	public synchronized void kill() {
	    killed = true;
	    notify();
	}

	/**
	 * This is the processing loop to pull data from a
	 * PullBufferDataSource.
	 */
	public void run() {
	    while (!killed) {
		try {
		    while (paused && !killed) {
			wait();
		    }
		} catch (InterruptedException e) {}

		if (!killed) {
		    boolean done = handler.readPullData(stream);
		    if (done)
			pause();
		}
	    }
	}
    }


}