package apollo.hermes;

/**

 * @author Konrad Rzeszutek (konrad@darnok.org)
 * @version $Id: JiniListner.java,v 1.2 2002/04/14 01:26:27 uid500 Exp $
 */

import net.jini.lease.*;
import net.jini.core.lookup.*;
import net.jini.discovery.*;
import net.jini.core.discovery.*;
import net.jini.core.event.*;
import net.jini.core.lease.*;
import java.rmi.*;
import java.rmi.server.*;
import java.io.*;

import apollo.common.jini.JiniServerComponentInfo;
import apollo.hermes.servers.Server;
import apollo.hermes.list.*;

/**
 * Jini component used to find the Jini services offered by all other servers.
 *
 * As explained in the API:
 * <br><br>
 * The RemoteEventListener interface needs to be implemented by any object
 * that wants to receive a notification of a remote event from some other object.
 * <br><br>
 * The object implementing this interface does not need to be the object that
 * originally registered interest in the occurrence of an event.
 * To allow the notification of an event's occurrence to be sent to an entity
 * other than the one that made the interest registration, the registration call
 * needs to accept a destination parameter, which indicates the object to
 * which the notification should be sent. This parameter must be an object
 * which supports the RemoteEventListener interface.
 *
 */
class JiniListner extends UnicastRemoteObject implements RemoteEventListener, Serializable{

    //Private Variable Declaration
    private ServiceTemplate template;
    private ServersList servers;

    public JiniListner(ServiceTemplate template,ServersList servers) throws java.rmi.RemoteException{
	this.template = template;
        this.servers =servers;
    }

    public void notify(RemoteEvent e) throws UnknownEventException,java.rmi.RemoteException{
/*
        Class[ ] classes = new Class[ ]{JiniHermesInfo.class, JiniDemeterInfo.class, JiniMorsInfo.class, JiniNemesisInfo.class};
        ServiceTemplate template = new ServiceTemplate(null, classes, null);
*/

	ServiceRegistrar registrar = (ServiceRegistrar)e.getSource( );


	try{
	    ServiceMatches services = registrar.lookup(template,Integer.MAX_VALUE);
	    // get the locator associated with this service
	    LookupLocator locator = registrar.getLocator( );
	    for(int j =0; j< services.items.length; j++){
		if(services.items[j].service != null){
                JiniServerComponentInfo info = (JiniServerComponentInfo)services.items[j].service;

                Server s = new Server(info.getRMIName(),info.getRMIHost(),info.getRMIPort());
                servers.addServer(s);

		}//end if
	    }//end for
	}catch(Exception ex){
            ex.printStackTrace();
	}

    }
}