package apollo.hermes.list;

/*
 * @author Konrad Rzeszutek (konrad@darnok.org)
 * @version 1.0
 */
import apollo.hermes.servers.*;
import apollo.hermes.events.*;
import java.util.*;
public class ServerTypeList {

  public static final int RETRY_TIME = 3;
  public static final int RETRY_TIMEOUT = 1000;
  private String type;
  private Vector v;

  /**
   * Constructs an empty list of servers with the specified type.
   */
  public ServerTypeList(String type) {
    this.type = type;
    v = new Vector(10);
  }

  /**
   * Adds the specified element to the list in a unspecified position.
   * The capacity of the list is increased by one.
   *
   * @param s the Server to be added
   */
  public void addElement(Server s) {

    v.add((Object) s);
  }

  /**
   * Gets a specific Server.
   *
   * @return i   index of the Server to return
   */
  public Server getElement(int i) throws ArrayIndexOutOfBoundsException{

    return ((Server)v.get(i));
  }

  /**
   * Removes a specific Server from the list
   * @param s the Server to be removed
   */
  public void removeElement(Server s) {

    v.remove((Object)s);

  }
  /**
   * Gets the size of the list of servers.
   */
  public int length() {

    return v.size();
  }
  /**
   * Gets the Server with the least load
   *
   * @return  if there are no servers on the list <code>null</code>. Otherwise
   *  a <code>Server</code> object is returned.
   */
  public Server getWithLeastLoad() {

   // Sort the list.
   Server s = null;

   if (v.size() > 0) {
      Collections.sort(v);
      s = getElement(0);
    }
    return s;
  }

  /**
   * Gets the specific type this List is.
   *
   * @return != null
   */

  public String getType() {
    return type;
  }

  /**
   * Retrieve the String representation of the list
   *
   */
  public String toString() {

    String z = "";
    if (v.size() > 0) {
      Collections.sort(v);
      for (int i = 0; i<length();i++)
        z+= getElement(i) +"\n";
    }
    return z;
  }

  /**
   * Refresh the list.
   *
   */
  public void refreshList() throws ServerUnreachable {


    for (int i = 0; i<length(); i++)
      try {
        getElement(i).getNrClients();
      } catch (Exception e) {
        //e.printStackTrace();
        /*
        int err = 0;
        Object waitTry = new Object();
        for (int j=0; j<RETRY_TIME; j++) {
          try {
            synchronized (waitTry) {
              waitTry.wait(RETRY_TIMEOUT);
            }
            // Try again after sleeping RETRY_TIMEO?UT
            //System.out.println("Trying .. "+getElement(i).getHost());
            getElement(i).getNrClients();
          }  catch (Exception error) {
            // Nope, problem..
            err ++;
          }
        }
        if (err > RETRY_TIME-1) {
          // Tried three times. Host is dead.*/
          String host = getElement(i).getHost();
          v.remove(i);
          throw new ServerUnreachable(host);
        //}
      }

  }
}