package apollo.hermes.rmi;

/**
 * @author Konrad Rzeszutek (konrad@darnok.org)
 * @version 1.0
 */

import java.rmi.*;
import java.rmi.server.UnicastRemoteObject;

public class ServerRMIInfo extends UnicastRemoteObject
                          implements ServerRMIInfoInterface{

  private int nr = -1;
  private int max = -1;
  private String name;
  private String ver;

  /**
   * Initializes the common Hermes information exchange RMI portal object
   *
   * @param max Max number of clients the server can handle
   * @param name  Name of the server
   * @param ver Version of the server
   */
  public ServerRMIInfo(int max, String name, String ver) throws RemoteException{

    nr = 0;
    this.max = max;
    this.name = name;
    this.ver = ver;
  }


  /**
   * Gets the max number of clients the server (on which this RMI
   * stub is running) can support.
   *
   * @return    >= 0
   *
   */
  public int getMaxClients() throws RemoteException {
    return max;
  }

  /**
   * Sets the max number of clients that the server can allow.
   *
   * @param   nr  >= 0
   */
  public void setMaxClients(int nr) throws RemoteException {
    max = nr;
  }

  /**
   * Sets the  number of clients that the server is currently serving.
   *
   * @param   nr  >= 0
   */
  public void setNrClients(int nr) throws RemoteException {
    this.nr = nr;
  }

  /**
   * Gets the current number of clients the server (on which this RMI
   * stub is running) is currently handling.
   *
   * @return    >= 0
   *
   */

  public int getNrClients() throws RemoteException {
    return nr;
  }

  /**
   * Gets the name of the server.
   *
   * @return	!= null
   */
  public String getName() throws RemoteException {
    return name;
  }

  /**
   * Sets the name of the server.
   *
   * @param  name  Name of the server.
   */
  //public void setName(String mame) throws RemoteException;

  /**
   * Gets the version of the server.
   *
   * @return != null
   */
  public String getVersion() throws RemoteException {
    return ver;
  }

  /**
   * Sets the version of the server.
   *
   * @param ver  Version of the server in string representation.
   */
  //public void setVersion(String ver) throws RemoteException;

}
