package apollo.hermes.rmi;

/**
 * @author Konrad Rzeszutek (konrad@darnok.org)
 * @version 1.0
 */
import java.rmi.Remote;
import java.rmi.RemoteException;

public interface ServerRMIInfoInterface extends Remote {



  /**
   * Gets the max number of clients the server (on which this RMI
   * stub is running) can support.
   *
   * @return    >= 0
   *
   */
  public int getMaxClients() throws RemoteException;

  /**
   * Sets the max number of clients that the server can allow.
   *
   * @param   nr  >= 0
   */
  public void setMaxClients(int nr) throws RemoteException;

  /**
   * Sets the  number of clients that the server is currently serving.
   *
   * @param   nr  >= 0
   */
  public void setNrClients(int nr) throws RemoteException;

  /**
   * Gets the current number of clients the server (on which this RMI
   * stub is running) is currently handling.
   *
   * @return    >= 0
   *
   */

  public int getNrClients() throws RemoteException;

  /**
   * Gets the name of the server.
   *
   * @return	!= null
   */
  public String getName() throws RemoteException;

  /**
   * Sets the name of the server.
   *
   * @param  name  Name of the server.
   */
  //public void setName(String mame) throws RemoteException;

  /**
   * Gets the version of the server.
   *
   * @return != null
   */
  public String getVersion() throws RemoteException;

  /**
   * Sets the version of the server.
   *
   * @param ver  Version of the server in string representation.
   */
  //public void setVersion(String ver) throws RemoteException;


}
