package apollo.mors;

/**
 * $Log: MorsDaemon.java,v $
 * Revision 1.8  2002/04/28 19:14:34  uid500
 * Final updates.
 *
 * Revision 1.7  2002/04/25 18:59:11  uid500
 * *** empty log message ***
 *
 * Revision 1.6  2002/04/16 19:25:54  uid500
 * Added input for RMI...
 *
 * Revision 1.5  2002/04/14 01:26:59  uid500
 * Added comments and added envionment variables
 *
 * Revision 1.4  2002/04/02 02:12:08  uid500
 * Modified
 *
 * Revision 1.3  2002/04/01 03:01:53  uid500
 * Took out stale code
 *
 * Revision 1.2  2002/03/31 03:23:18  uid500
 * *** empty log message ***
 *
 * Revision 1.1.1.1  2002/03/30 18:51:33  uid500
 * Apollo src code import
 *
 *
 * @author Konrad Rzeszutek (konrad@darnok.org)
 * @version $Id: MorsDaemon.java,v 1.8 2002/04/28 19:14:34 uid500 Exp $
 */
import apollo.common.*;
import apollo.common.gui.*;
import apollo.common.tui.*;
import apollo.common.jini.*;
import apollo.common.client.*;
import apollo.hermes.rmi.*;
import apollo.hermes.servers.*;
import apollo.mors.rmi.*;
import apollo.mors.jini.*;
import java.rmi.*;
import java.rmi.server.*;
import java.net.*;
import java.io.*;
import net.jini.core.lookup.*;
/**
 * Mors (aka Thantos) personification of death.
 *
 * This daemon's task is to receive the events signifying a face-detection
 * event, save it locally and then proudly display it to the console.
 *
 * This class is the starter object which starts the GUI, RMI, Jini and ties
 * them all together.
 *
 */
public class MorsDaemon implements FaceDetectionEventListener, Daemon{

  private final static String DAEMON_NAME  = "MorsD";
  private final static String DAEMON_VER   = "$Id: MorsDaemon.java,v 1.8 2002/04/28 19:14:34 uid500 Exp $";

  /**
   * Max number of clients we can handle.
   */
  public static int DAEMON_MAX_CLIENTS = 64;
  private final static int THREAD_WAIT_TIME = 500;

  private MorsEventGenerator mors_e;
  private MorsRMIServer mors_rmi;
  private JiniServerRegister mors_register;
  private ServerRMIInfo hermes_rmi;
  private JiniMorsInfo mors_jini;
  private Log log = new TextLog();
  private apollo.common.gui.MainFrame frame;
  private String hermes_url;
  private String mors_url;
  private String ver;
  private ClientList clients;
  private String RMI_server;

  private int RMI_port;
  private boolean stopThread = false;
  private Object waitThread = new Object();


  /**
   * Start the MorsDaemon.
   *
   * @param nogui If the system should run without a gui
   */
  public MorsDaemon(boolean nogui, String MORS_DIR) {


    // Find out about ourselves - what's our IP, where the RMI server is,
    // on what port it resides
    getHostInfo();


    hermes_url = "rmi://"+RMI_server+":"+RMI_port+"/"+
      JiniServerComponentInfo.MORS;
    mors_url = "rmi://"+RMI_server+":"+RMI_port+"/"+ MorsRMIInterface.MORS_RMI;


    if (!nogui) {
      frame = new MainFrame(DAEMON_NAME, this);
      frame.setSize(800,600);
      frame.validate();
      frame.setVisible(true);
      // Wait until its up?

      Object waitFrame = new Object();
      while (frame.log == null) {
        synchronized (waitFrame) {
          try {
            waitFrame.wait(1000);
          } catch (InterruptedException e) { }
        }
      }
      log = ((Log)frame.log);
    }

    // Start the Hermes RMI server
    System.setSecurityManager(new RMISecurityManager());

    try {
      hermes_rmi = new ServerRMIInfo(DAEMON_MAX_CLIENTS,mors_url,ver);
      mors_rmi = new MorsRMIServer();
      mors_rmi.registerServerRMIInfo(hermes_rmi);
    } catch (RemoteException e) { e.printStackTrace(); }


    // Bind RMI
    try {
          log.println("Rebinding "+hermes_url,Log.INFO);
          Naming.rebind(hermes_url,hermes_rmi);
          log.println("Rebinding "+mors_url,Log.INFO);
          Naming.rebind(mors_url,mors_rmi);
    } catch (Exception  e) {
      if (log.fatalError(e.getMessage())) exitDaemon();
      // Couldn't bind to RMI. Fatal error.
    }

    log.println("Started "+hermes_url+" server ..",Log.INFO);
    log.println("Started "+mors_url+" server ..",Log.INFO);

    // Start the Jini server

    mors_jini = new JiniMorsInfo(RMI_server, RMI_port);
    mors_register = new JiniServerRegister(
      new ServiceItem(null,mors_jini,null),log);
    log.println("Started Jini server ..",Log.INFO);


    // Start the Event Generator
    mors_e = new MorsEventGenerator(mors_rmi, log,MORS_DIR);
    if (frame != null)
      frame.registerFaceDetectionEventGenerator(mors_e);

    // Link events to prospective listeners.
    mors_e.addFaceDetectionEvent(this);
    if (frame != null) {
      mors_e.addFaceDetectionEvent(frame);
      frame.clients = new ClientListFrame();
      frame.addJInternalFrame(frame.clients);
      mors_rmi.registerClientListFrame(frame.clients);
    }

    log.println("Daemon started. ",Log.INFO);
  }

  public void faceDetectionEventReceived(FaceDetectionEvent event) {

    if (frame == null)
      log.println(event.toString(),event.getHost());
  }


  /**
   * Really bad stuff happens here. We close everything we can
   * cleanly and then exit.
   */
  public void exitDaemon() {

    stopThread = true;
    // Most important are the threads. We want to make sure that they
    // release everything and save data. Then we can kill them
    if (mors_e != null) mors_e.killThread();
    try {
      Naming.unbind(mors_url);
      Naming.unbind(hermes_url);
    } catch (Exception e) { } // Ignore

    frame = null;
    System.exit(0);
  }


  /**
   * Find out what our IP is. Where the RMI server resides, and what port
   * is the RMI server up on. These info are assumed to be the local machine,
   * but if the RMI server is located somewhere else, the
   *  RMISERVER variable and RMIPORT hold the values.
   */
  public void getHostInfo() {

      try {
        RMI_server = System.getProperty("RMISERVER",
          java.net.InetAddress.getLocalHost().getHostAddress());

      } catch (Exception e) {
        if (log.fatalError(e.getMessage())) exitDaemon();
      }
      String tmp = System.getProperty("RMIPORT","1098");
      RMI_port = 1098;
      try {
        RMI_port = Integer.parseInt(tmp);
      } catch (Exception e) { }

  }

  private static void prUsage() {
           System.out.println("java -jar mors [-nogui] -d directory");
                System.exit(-2);

  }
  public static void main(String[] args) {

    int i = 0;
    boolean nogui = false;
    String dir = null;
    while (i < args.length) {
      if (args[i].equals("-nogui")) {
		i++;
                nogui = true;
      } else if (args[i].equals("-h")) {
        prUsage();
      } else if (args[i].equals("-d")) {
          i++;
          if (i >= args.length)
            prUsage();
          dir = args[i];
      }
      i++;
    }

    if (dir == null)
      prUsage();

    new MorsDaemon(nogui, dir);

  }
}
