package apollo.mors;

/**
 * $Log: MorsEventGenerator.java,v $
 * Revision 1.6  2002/04/28 19:14:34  uid500
 * Final updates.
 *
 * Revision 1.5  2002/04/16 19:25:54  uid500
 * Added input for RMI...
 *
 * Revision 1.4  2002/04/14 01:26:59  uid500
 * Added comments and added envionment variables
 *
 * Revision 1.3  2002/04/01 03:01:53  uid500
 * Took out stale code
 *
 * Revision 1.2  2002/03/31 03:23:18  uid500
 * *** empty log message ***
 *
 * Revision 1.1.1.1  2002/03/30 18:51:33  uid500
 * Apollo src code import
 *
 *
 * Copyright:    Copyright (c) 2002
 * Company:      www.darnok.org
 * @author Konrad Rzeszutek (konrad@darnok.org)
 * @version $Id: MorsEventGenerator.java,v 1.6 2002/04/28 19:14:34 uid500 Exp $
 */

import java.util.*;
import java.io.*;
import apollo.common.*;
import apollo.mors.rmi.*;
/**
 * Generator of FaceDetection Events. The events are picked up from MorsRMIServer
 * and dispatched to the EventListners.
 */
public class MorsEventGenerator extends Thread implements
  FaceDetectionEventGenerator {

  private MorsRMIServer mors;
  private ArrayList listners = new ArrayList();
  private FaceDetectionEvent e;
  public String EVENT_DIR;

  // For Thread part:
  private boolean stopThread = false;
  private boolean dont_save = false;
  private Object waitThread = new Object();
  private Log log;
  private static final int THREAD_SLEEP_TIME = 1000;  // Check every second
  private ArrayList events = new ArrayList();

  /**
   * Initialize the generator firring off events FAceDetection events to
   * listners.
   *
   * @param mors  The RMI server which receives the FaceDetectionEvent's from
   *  the clients
   */
  public MorsEventGenerator(MorsRMIServer mors, Log log,String DIR) {

    this.mors = mors;
    this.log = log;
    this.EVENT_DIR=DIR;
    this.start();
  }

  /**
   * Registers a listener for the FaceDetectionEvents.
   *
   *
   */
  public synchronized void addFaceDetectionEvent(FaceDetectionEventListener l) {

    listners.add(l);
  }

  /**
   * Unregisters a listener from the FaceDetectionEvents.
   */
  public synchronized void removeFaceDetectionEvent(
    FaceDetectionEventListener l) {

    // Will this crap out if I submit a non-registered listener?
    listners.remove(l);
  }

  /**
   * Opens up a saved FaceDetectionEvent and notifies listeners of a new
   * event.
   *
   * @param fileName  Full location of the file
   * @exception IOException if there are problems reading the event
   * @exception ClassNotFoundException problems unmarshelling - wrong version of
   *  FaceDetectionEvent
   * @exception FileNotFoundException - pretty much self-explanatory.
   */
  public synchronized void readEvent(String fileName) throws IOException,
      ClassNotFoundException,FileNotFoundException {

    FileInputStream i_stream = new FileInputStream(fileName);
    ObjectInputStream p = new ObjectInputStream(i_stream);

      // Put on queue of Events.
    events.add (p.readObject());
    dont_save = true;
    i_stream.close();
  }

  /**
   * Save the event. The information where to save is determined from
   * the FaceDetectionEvent.getUniqueuName().
   */
  private void saveEvent() throws IOException {

    createDirectoryFromFile(EVENT_DIR + "/" + e.getUniqueName());
    FileOutputStream o_stream = new FileOutputStream(EVENT_DIR + "/"
      +e.getUniqueName());
    ObjectOutputStream p = new ObjectOutputStream(o_stream);
    log.println("Saving: "+e.getUniqueName(),Log.INFO);
    p.writeObject(e);
    o_stream.close();

  }

  /**
   * Helper function creating the root directory (if it doesn't exist)
   * from the given <code>dir</code>
   *
   * @param f The string for the file from which to create the directory
   */
  private void createDirectoryFromFile (String f) {

        File[] dirs = null;
        File top = new File(f);
        String step = top.getAbsolutePath();
        int cnt = 0;

        for (int i = 0; i< step.length(); i++)
          if (step.charAt(i) == '\\') cnt++;

        dirs = new File[cnt];
        dirs[dirs.length-1] = top.getParentFile();
        for (int i = dirs.length-2; i>=0; i--)
          dirs[i] = new File(dirs[i+1].getParent());

        for (int i = 0; i< dirs.length; i++) {
          //System.out.println(dirs[i].getAbsolutePath());
          dirs[i].mkdir();
        }
  }

  /**
   * Fires off the event to the listners.
   */
  private synchronized void fireFaceDetectionEvent() {

    FaceDetectionEvent event = new FaceDetectionEvent( this,
        e.getHost(), e.getLocation(), e.getDate(), e.getDescription(),
        e.getCameraImage(), e.getMatchedImage());

    Iterator i = listners.iterator();

    while ( i.hasNext() ) {
      ((FaceDetectionEventListener)i.next() ).faceDetectionEventReceived(event);
    }
  }

  /**
   * The main part of this object. It iterates thru the list of events
   * from MorsRMIServer and takes one, saves it locally, and then pushes
   * it out to the listners.
   *
   */
  public void run () {

    while ( !stopThread ) {

        // Get the event from FIFO
        e = mors.getLastEvent();
        if (e == null) {
          // Get some event from the event queueu (if we have some)
          if (!events.isEmpty()) {
            e = (FaceDetectionEvent)events.get(0);
            events.remove(0);
          }
        }

        if (e != null) {
          // Just save it.
          try {
            if (!dont_save)
              saveEvent();
            else
              dont_save = false;

          } catch (IOException e) {
          // This is terrible. Wrong path + name and can't create the
          // directory.
            log.println(e.getMessage(), Log.ERROR);
          }
          fireFaceDetectionEvent();
        }
        e = null;


      // Sleep.
      synchronized (waitThread) {
        try {
          waitThread.wait(THREAD_SLEEP_TIME);
        } catch (InterruptedException er) { }
      }
    }
  }

  /**
   * Safely stops the thread.
   */
  public void killThread() {

    stopThread = true;
    synchronized (waitThread) {
      try {
        waitThread.notifyAll();
       } catch (Exception er) { }
    }
  }

}
