package apollo.mors.rmi;

/**
 * Title:        Apollo - Dynamic Distributed Face Recognition Security System
 * Description:  #replace#
 *
 * $Log: MorsRMIServer.java,v $
 * Revision 1.7  2002/04/28 19:14:36  uid500
 * Final updates.
 *
 * Revision 1.6  2002/04/16 19:25:56  uid500
 * Added input for RMI...
 *
 * Revision 1.5  2002/04/14 01:27:03  uid500
 * Added comments
 *
 * Revision 1.4  2002/04/02 02:12:09  uid500
 * Modified
 *
 * Revision 1.3  2002/04/01 03:01:57  uid500
 * Took out stale code
 *
 * Revision 1.2  2002/03/31 03:23:42  uid500
 * *** empty log message ***
 *
 * Revision 1.1.1.1  2002/03/30 18:51:33  uid500
 * Apollo src code import
 *
 *
 * Copyright:    Copyright (c) 2002
 * Company:      www.darnok.org
 * @author Konrad Rzeszutek (konrad@darnok.org)
 * @version $Id: MorsRMIServer.java,v 1.7 2002/04/28 19:14:36 uid500 Exp $
 */
import java.rmi.*;
import java.rmi.server.*;
import apollo.common.gui.ClientListFrame;
import apollo.common.client.*;
import apollo.hermes.rmi.*;
import apollo.mors.rmi.*;
import apollo.common.*;
import apollo.common.events.*;
import apollo.mors.*;
import java.util.*;

/**
 * This service recevies the events confirming a face-recognition match.
 *
 * This class is the implementaiton of the transportation layer for such events. First
 * the client has to register, then it is allowed to send events.
 */

public class MorsRMIServer extends UnicastRemoteObject
                          implements MorsRMIInterface {

  private ClientList clients;
  private Vector events;
  private ClientListFrame frame;
  private ServerRMIInfo rmi;

  public MorsRMIServer() throws RemoteException {

    clients = new ClientList();
    events = new Vector(32);
  }


    /**
     * Sends an event to the Mors server. The client sending it has to be
     * registered, otherwise the event is discarded.
     *
     * @throws IsNotRegisteredException if the autor of the <code>c</c> has not
     * been registered
     */
    public void sendEvent(FaceDetectionEvent e, Client ip) throws
                RemoteException {

        // Check to make sure that the Event came from a registered IP
        int ii = 0;

        for (ii = 0; ii < clients.length() &&
          (((Client)clients.getElement(ii)).getHost().compareToIgnoreCase(
            e.getHost()) != 0); ii++);


        if (ii == clients.length())
          clients.addElement(ip);

        if ( (clients.length() == 0)  ||
          (((Client)clients.getElement(ii)).getHost().compareToIgnoreCase(
            e.getHost()) != 0) )
          clients.addElement(ip);
        // Otherwise add the event to our list

        events.add((Object)e);
        clients.getElement(ii).addEvent();

        // Call ServerRMI to update the count.
        rmi.setNrClients(clients.length());
        this.refreshList();
    }

    /**
     * Register part of the GUI component with this service. This is important
     * for the refreshList() method to work.
     *
     * @param f The ClientListFrame GUI
     */
    public void registerClientListFrame(ClientListFrame f) {

      this.frame = f;
    }

    /**
     * Registers the Server RMI (for Hermes) which is used to set the number
     * of clients using this service
     *
     * @param p The ServerRMIInfo object
     */
     public void registerServerRMIInfo(ServerRMIInfo p) {

      this.rmi = p;
     }

    /**
     * Refresh the list used by the GUI. To be called after adding the
     * client to the list of clients.
     */
    private void refreshList() {

      if (frame != null)
        frame.refresh(clients);
    }

    /**
     * Gets the event and clear it from the list of events submitted. The list
     * is treated like a FIFO stack
     *
     * @return null or an element
     */
     public FaceDetectionEvent getLastEvent() {

      FaceDetectionEvent a = null;

      if (events.size() > 0) {

        // Gets the FIFO element
        a = (FaceDetectionEvent)events.get(0);
        events.remove(0);
      }

      return a;
     }

      public void isConnectionOk() throws RemoteException { }
}
