package apollo.nemesis;
import apollo.common.*;
import apollo.common.gui.*;
import apollo.common.tui.*;
import apollo.common.jini.*;
import apollo.common.client.*;
import apollo.hermes.rmi.*;
import apollo.hermes.jini.*;
import apollo.common.movie.*;
import apollo.hermes.servers.*;
import apollo.nemesis.rmi.*;
import apollo.nemesis.jini.*;
import apollo.demeter.save.*;
import apollo.common.save.*;
import apollo.demeter.client.*;
import java.rmi.*;
import java.rmi.server.*;
import java.net.*;
import java.io.*;
import net.jini.core.lookup.*;
import javax.media.*;
import java.rmi.*;
import java.rmi.server.*;
import net.jini.lease.*;
import net.jini.core.lookup.*;
import net.jini.discovery.*;
import net.jini.core.discovery.*;
import net.jini.core.event.*;
import net.jini.core.lease.*;
import net.jini.core.lookup.ServiceItem;
/**
 * Title:
 * Description:
 * Copyright:
 * Company:
 * @author Konrad Rzeszutek (konrad@darnok.org)
 * @version $Id: NemesisDaemon.java,v 1.4 2002/04/28 19:14:37 uid500 Exp $
 */
/*


 */
public class NemesisDaemon implements Daemon, DiscoveryListener,
   FaceDetectionEventListener{

  private final static String DAEMON_NAME  = "NemesisD";
  private final static String DAEMON_VER   = "$Id: NemesisDaemon.java,v 1.4 2002/04/28 19:14:37 uid500 Exp $";

  /**
   * Max amounts of clietns we can handle.
   */
  public static int DAEMON_MAX_CLIENTS = 4;

  private NemesisRMIServer nemesis_rmi;
  private JiniServerRegister nemesis_register;
  private NemesisEventGenerator nemesis_e;
  private ServerRMIInfo hermes_rmi;
  private JiniNemesisInfo nemesis_jini;
  private Log log = new TextLog();
  private MainFrame frame;
  private String hermes_url;
  private String nemesis_url;
  private String mors_url;
  private LookupDiscovery lookup              = null;
  private String ver;
  //private ClientList clients;
  private String RMI_server;
  private int RMI_port;
  private Queen queen;
  //private NemesisEventGenerator finder;

  /**
   * Start the MorsDaemon.
   *
   * @param nogui If the system should run without a gui
   * @param dir Directory where images and movies need to be stored.
   */
  public NemesisDaemon(boolean nogui, String dir, String data_dir) {


    // Find out about ourselves - what's our IP, where the RMI server is,
    // on what port it resides
    getHostInfo();

    hermes_url = "rmi://"+RMI_server+":"+RMI_port+"/"+ JiniServerComponentInfo.NEMESIS;
    nemesis_url = "rmi://"+RMI_server+":"+RMI_port+"/"+ NemesisRMIInterface.NEMESIS_RMI;

    if (!nogui) {
      frame = new MainFrame(DAEMON_NAME, this);
      frame.setSize(800,600);
      frame.validate();
      frame.setVisible(true);

      Object waitFrame = new Object();
      while (frame.log == null) {
        synchronized (waitFrame) {
          try {
            waitFrame.wait(1000);
          } catch (InterruptedException e) { }
        }
      }
      log = ((Log)frame.log);
    }

    // Link events to prospective listeners.
    if (frame != null) {
      frame.clients = new ClientListFrame();
      frame.addJInternalFrame(frame.clients);

    }
    nemesis_e = new NemesisEventGenerator(log,dir);
    nemesis_e.addFaceDetectionEvent(this);


    FaceRecognition face_rec = new FaceRecognition(log, this);

    if (frame != null) {
      nemesis_e.addFaceDetectionEvent(frame);
      frame.addJInternalFrame(face_rec.getComponent());
    }

    face_rec.constructFacesFrom(data_dir);

    if (frame != null)
      queen = new Queen(log, nemesis_e, face_rec, dir, frame.clients);
    else
      queen = new Queen(log, nemesis_e, face_rec, dir, null);



    // Start the Hermes RMI server
    //  System.setSecurityManager(new RMISecurityManager()); /* This breaks JMF */

    try {
      hermes_rmi = new ServerRMIInfo(DAEMON_MAX_CLIENTS,nemesis_url,ver);
      nemesis_rmi = new NemesisRMIServer();
    } catch (RemoteException e) { e.printStackTrace();
      log.normalError(e.getMessage());
    }


    // Bind RMI
    try {
          log.println("Rebinding "+hermes_url,Log.INFO);
          Naming.rebind(hermes_url,hermes_rmi);
          log.println("Rebinding "+nemesis_url,Log.INFO);
          Naming.rebind(nemesis_url,nemesis_rmi);
    } catch (Exception  e) {
    e.printStackTrace();
      if (log.fatalError(e.getMessage())) exitDaemon();
      // Couldn't bind to RMI. Fatal error.
    }

    log.println("Started "+hermes_url+" server ..",Log.INFO);
    log.println("Started "+nemesis_url+" server ..",Log.INFO);

    // Start the Jini server

    nemesis_jini = new JiniNemesisInfo(RMI_server, RMI_port);
    nemesis_register = new JiniServerRegister(
      new ServiceItem(null,nemesis_jini,null),log);
    log.println("Started Jini server ..",Log.INFO);


    nemesis_rmi.registerQueen(queen);
    queen.registerServerRMIInfo(hermes_rmi);

        // Start our Jini listner for Hermes
    try{
      lookup = new LookupDiscovery(LookupDiscovery.ALL_GROUPS);
    }catch(java.io.IOException e){
        if (log.fatalError(e.getMessage())) exitDaemon();
    }

    lookup.addDiscoveryListener(this);

    log.println("Daemon started. ",Log.INFO);

}
  /**
   * Really bad stuff happens here. We close everything we can
   * cleanly and then exit.
   */
  public void exitDaemon() {

    nemesis_e.killThread();
    try {
      Naming.unbind(nemesis_url);
      Naming.unbind(hermes_url);
    } catch (Exception e) { } // Ignore them
    // Kill the GUI
    frame = null;

    System.exit(0);
  }

   /**
   * Find out what our IP is. Where the RMI server resides, and what port
   * is the RMI server up on. These info are assumed to be the local machine,
   * but if the RMI server is located somewhere else, the
   *  RMISERVER variable and RMIPORT hold the values.
   */
  public void getHostInfo() {

      try {
        RMI_server = System.getProperty("RMISERVER",
          java.net.InetAddress.getLocalHost().getHostAddress());

      } catch (Exception e) {
        if (log.fatalError(e.getMessage())) exitDaemon();
      }
      String tmp = System.getProperty("RMIPORT","1098");
      RMI_port = 1098;
      try {
        RMI_port = Integer.parseInt(tmp);
      } catch (Exception e) { }


  }
  /**
   * Called when one or more lookup service registrars has been discovered.
   * The method should return quickly; e.g., it should not make remote calls.
   *
   * @param e - the event that describes the discovered registrars
   */
  public synchronized void discovered(DiscoveryEvent e){

    ServiceTemplate hermes = new ServiceTemplate(null,
       new Class[]{JiniHermesInfo.class}, null);
    ServiceRegistrar[ ] registrars = e.getRegistrars( );

    for(int i =0; i< registrars.length; i++){

      try{
          ServiceRegistrar registrar = registrars[i];
          log.println("Discovered "+registrar.getLocator().getHost()+
            " Jini server!","Jini");
          checkJiniFor(hermes,  registrars[i]);
      }catch(Exception ex){
          log.normalError(ex.getMessage());
        ex.printStackTrace();

	    }//end catch
	}//end outer for
    }//end method discovered

  /**
   * Checks the Jini for the <code>template</code> type server with the
   * given <code>registrar</code> and registers a JiniListner in case
   * the service appears at some point later on.
   *
   */
    public void checkJiniFor(ServiceTemplate template,ServiceRegistrar registrar)
     throws RemoteException{

        ServiceMatches services = registrar.lookup(template,Integer.MAX_VALUE);

        LookupLocator locator = registrar.getLocator( );

	/**
	 *  get the information from each one of the services
	 */
	for(int j =0; j< services.items.length; j++){
	  if(services.items[j].service != null){
                JiniServerComponentInfo info =
                (JiniServerComponentInfo)services.items[j].service;
                nemesis_e.newRegistrar(info);
		}//end if
	    }//end for
          EventRegistration ereg = registrar.notify(template,
		ServiceRegistrar.TRANSITION_NOMATCH_MATCH,
		new JiniListner(template, nemesis_e),
		null,
                Lease.ANY);

    }

    /**
     * Called when one or more lookup service registrars has been discarded.
     *
     * @param e - the event that describes the discarded registrars
     */
    public synchronized void discarded(DiscoveryEvent e){
	ServiceRegistrar[] registrars = e.getRegistrars( );
	for(int i=0; i<registrars.length;i++){
	/**
	 * a condition where a Service Registrar
	 * leaves the vicinity.We cannot move the
	 * users from the lookup server becuase they
	 * may actually be present on the other
	 * lookup servers.
	 */
         try{
 		ServiceRegistrar registrar = registrars[i];
		LookupLocator locator = registrar.getLocator( );
		log.println("The Lookup Service "+locator.getHost( )
                  +" has left.","Jini");
	    }catch(Exception ex){

                //ex.printStackTrace();
                log.println("Registrar could not leave","Jini");
	    }
	}//end for
    }//end method

  /**
   * Receiver for the event.
   *
   * @param event Is guaranteed not to be null.
   */
  public void faceDetectionEventReceived(FaceDetectionEvent event) {

    log.println(event.toString(),Log.INFO);
    // Dont want it to pop up?
  }

  private static void prUsage() {
    System.out.println("java -jar nemesis [-nogui] -d directory -f faces_directory");
    System.exit(-2);

  }
  public static void main(String[] args) {

    int i = 0;
    boolean nogui = false;
    String dir = null;
    String data_dir = null;

    while (i < args.length) {
      if (args[i].equals("-nogui")) {
		i++;
                nogui = true;
      } else if (args[i].equals("-h")) {
        prUsage();
      } else if (args[i].equals("-d")) {
          i++;
          if (i >= args.length)
            prUsage();
          dir = args[i];
      } else if (args[i].equals("-f")) {
          i++;
          if (i >= args.length)
            prUsage();
          data_dir = args[i];
      }
      i++;
    }

    if ((data_dir == null) || (dir == null))
      prUsage();
    new NemesisDaemon(nogui, dir, data_dir);

  }
}
