package apollo.nemesis;

import apollo.common.*;
import apollo.common.events.*;
import apollo.common.jini.*;
import apollo.common.client.*;
import apollo.mors.rmi.*;
import apollo.hermes.rmi.*;
import apollo.nemesis.*;
import apollo.nemesis.client.*;
import java.util.*;
import java.io.*;
import java.rmi.*;
/**
 * @author Konrad Rzeszutek (konrad@darnok.org)
 * @version $Id: NemesisEventGenerator.java,v 1.3 2002/04/28 19:14:37 uid500 Exp $
 */

/**
 * Dispatches the Event's to listners. Also to the itself
 */
public class NemesisEventGenerator extends Thread implements
  FaceDetectionEventGenerator {

  //private MorsRMIServer mors;
  private ArrayList listners = new ArrayList();
  private FaceDetectionEvent e = null;
  private Client c = null;
  public String EVENT_DIR;

  // For Thread part:
  private boolean stopThread = false;
  private boolean dont_save = false;
  private Object waitThread = new Object();
  private Log log;
  private static final int THREAD_SLEEP_TIME = 1000;  // Check every second
  private ArrayList events = new ArrayList();
  private ArrayList clients = new ArrayList();
  private String mors_url = null;
  private String hermes_url = null;
  private ArrayList hermes_list = new ArrayList();

  public NemesisEventGenerator(Log log, String dir) {

    this.log = log;
    this.EVENT_DIR = dir;
    this.start();

  }

  public void newRegistrar(JiniServerComponentInfo info) {


    // Add to our list.
    log.println("Adding a new Hermes.",Log.INFO);
    hermes_list.add(info);
    // Check our current connection, with mors
    if (mors_url != null) {
      try {
        MorsRMIInterface mors = (MorsRMIInterface)Naming.lookup(mors_url);

      }  catch (Exception e) {
        log.println(mors_url+" is broken. ",Log.INFO);
        mors_url = null;
      }
    }
    if (mors_url == null) /* Connection is broken */ {
        mors_url = getMors();
    }

  }
  private String getMors() {

    String answer = null;
    JiniServerComponentInfo info = null;
    log.println("Finding a new Mors [last address: "+mors_url+"]", Log.INFO);

    for (int i = 0; i < hermes_list.size(); i++) {
      try {
      // Establish connection with Hermes
        info = (JiniServerComponentInfo)hermes_list.get(i);
        String h_url = "rmi://"+info.getRMIHost()+":"+info.getRMIPort()+"/"+
          info.getRMIName();
        HermesRMIInterface h = (HermesRMIInterface)Naming.lookup(h_url);
        answer = h.getURLForServerType(JiniServerComponentInfo.MORS);
        if (answer != null && !answer.equals("UNKNOWN")){
          answer += MorsRMIInterface.MORS_RMI;
          log.println("Found: "+answer,Log.INFO);
        }
      }  catch (Exception e) {
        log.println("Couldn't find Mors. Retrying later.",Log.ERROR);
        //e.printStackTrace();
      }
      /* We break from the searching whenever we have found our Mors */
      if (answer != null) break;
    }
    return answer;
  }

  public synchronized void submit(NemesisClient c, byte[] camera, byte[] m) {


    FaceDetectionEvent event = new FaceDetectionEvent( this,
        c.getHost(), c.getName(), new java.util.Date().toString(),
          c.getDescription(),
        camera,m);

    this.c = new Client(c.getHost(), c.getName());
    events.add(event);
    clients.add(this.c);
  }


 /**
   * Registers a listener for the FaceDetectionEvents.
   *
   *
   */
  public synchronized void addFaceDetectionEvent(FaceDetectionEventListener l) {

    listners.add(l);
  }

  /**
   * Unregisters a listener from the FaceDetectionEvents.
   */
  public synchronized void removeFaceDetectionEvent(
  FaceDetectionEventListener l) {

    // Will this crap out if I submit a non-registered listener?
    listners.remove(l);
  }

  /**
   * Opens up a saved FaceDetectionEvent and notifies listeners of a new
   * event.
   *
   * @param fileName  Full location of the file
   * @exception IOException if there are problems reading the event
   * @exception ClassNotFoundException problems unmarshelling - wrong version of
   *  FaceDetectionEvent
   * @exception FileNotFoundException - pretty much self-explanatory.
   */
  public synchronized void readEvent(String fileName) throws IOException,
      ClassNotFoundException,FileNotFoundException {

    FileInputStream i_stream = new FileInputStream(fileName);
    ObjectInputStream p = new ObjectInputStream(i_stream);

      // Put on queue of Events.
    FaceDetectionEvent event = (FaceDetectionEvent)p.readObject();
    events.add (event);
    clients.add(new Client(event.getHost(), event.getLocation()));
    dont_save = true;
    i_stream.close();
  }

  /**
   * Save the event. The information where to save is determined from
   * the FaceDetectionEvent.getUniqueuName().
   */
  private void saveEvent() throws IOException {

    createDirectoryFromFile(EVENT_DIR + "/" + e.getUniqueName());
    FileOutputStream o_stream = new FileOutputStream(EVENT_DIR + "/"
      +e.getUniqueName());
    ObjectOutputStream p = new ObjectOutputStream(o_stream);
    log.println("Saving: "+e.getUniqueName(),Log.INFO);
    p.writeObject(e);
    o_stream.close();

  }

  /**
   * Helper function creating the root directory (if it doesn't exist)
   * from the given <code>dir</code>
   *
   * @param f The string for the file from which to create the directory
   */
  private void createDirectoryFromFile (String f) {

        File[] dirs = null;
        File top = new File(f);
        String step = top.getAbsolutePath();
        int cnt = 0;

        for (int i = 0; i< step.length(); i++)
          if (step.charAt(i) == '\\') cnt++;

        dirs = new File[cnt];
        dirs[dirs.length-1] = top.getParentFile();
        for (int i = dirs.length-2; i>=0; i--)
          dirs[i] = new File(dirs[i+1].getParent());

        for (int i = 0; i< dirs.length; i++) {
          //System.out.println(dirs[i].getAbsolutePath());
          dirs[i].mkdir();
        }
  }

  /**
   * Fires off the event to the listners.
   */
  private synchronized void fireFaceDetectionEvent() {

    FaceDetectionEvent event = new FaceDetectionEvent( this,
        e.getHost(), e.getLocation(), e.getDate(), e.getDescription(),
        e.getCameraImage(), e.getMatchedImage());

    Iterator i = listners.iterator();

    while ( i.hasNext() ) {
      ((FaceDetectionEventListener)i.next() ).faceDetectionEventReceived(event);
    }

    // And now fire off to the RMI server for Mors.. How to keep it dynamic?
    //System.setSecurityManager(new RMISecurityManager());

    if (mors_url != null) {
    try {

      MorsRMIInterface m = (MorsRMIInterface)Naming.lookup(mors_url);
      m.sendEvent(event, c); // Where is the client from? What about more of them?

    } catch (Exception e) {
      //e.printStackTrace();
      log.println("No sending of events to Mors.",Log.ERROR);
    }
    } else {
      log.println("We don't have the address of Mors!",Log.ERROR);
    }

  }

  /**
   * The main part of this object. It iterates thru the list of events
   * from MorsRMIServer and takes one, saves it locally, and then pushes
   * it out to the listners.
   *
   */
  public void run () {
    int count = 0;
    while ( !stopThread ) {


          // Get some event from the event queueu (if we have any)
        if (!events.isEmpty()) {
            e = (FaceDetectionEvent)events.get(0);
            events.remove(0);
            c = (Client)clients.get(0);
            clients.remove(0);
        }
        if (e != null) {
          // Just save it.
          try {
            if (!dont_save)
              saveEvent();
            else
              dont_save = false;

          } catch (IOException e) {
          // This is terrible. Wrong path + name and can't create the
          // directory.
            log.println(e.getMessage(), Log.ERROR);
          }
          fireFaceDetectionEvent();
        }
        e = null;
        count++;
        if (count > 10) {
          count = 0;
          if (mors_url != null) {
            try {
              MorsRMIInterface mors = (MorsRMIInterface)Naming.lookup(mors_url);
              mors.isConnectionOk();
            } catch (Exception e) { mors_url = null; }
          }
          if (mors_url == null) /* Connection is broken */ {
              mors_url = getMors();
          }
        }

        // Sleep.
      synchronized (waitThread) {
        try {
          waitThread.wait(THREAD_SLEEP_TIME);
        } catch (InterruptedException er) { }
      }
    }
  }

  /**
   * Safely stops the thread.
   */
  public void killThread() {

    stopThread = true;
    synchronized (waitThread) {
      try {
        waitThread.notifyAll();
       } catch (Exception er) { }
    }
  }

}
