package apollo.nemesis;
import javax.media.*;
import javax.media.format.*;
import apollo.common.client.*;
import apollo.nemesis.client.*;
import java.rmi.*;
import javax.media.*;
import java.util.*;
import apollo.common.*;
import apollo.common.gui.*;
import apollo.hermes.rmi.ServerRMIInfo;
/**

 * @author Konrad Rzeszutek (konrad@darnok.org)
 * @version $Id: Queen.java,v 1.2 2002/04/28 19:14:38 uid500 Exp $
 */

/**
 * The service component responsible for synchronizing the
 * GUI, event generator, face recognition engine and spawing
 * two threads for the client - to save locally and to look
 * thru the files and run the face-recognition engine on the picture.
 */
public class Queen {


  private ClientList clients = new ClientList();
  private Log log;
  private NemesisEventGenerator g;
  private FaceRecognition f;
  private String dir;
  private ClientListFrame l;
  private ServerRMIInfo rmi;
  /**
   * Initialize the Hive Queen
   *
   * @param log The logging facility
   * @param g The Event generator which broadcasts the events to its listners
   * @param f the Face recognition engine
   * @param dir The directroy where the data will be saved
   * @param l The GUI representing the list of clients (updated accordingly as well)
   */
  public Queen(Log log, NemesisEventGenerator g, FaceRecognition f, String dir,
     ClientListFrame l) {

    this.log = log;
    this.f = f;
    this.g = g;
    this.dir = dir;
    this.l = l;
  }
  public void register(MediaLocator url, Client c) throws RemoteException {

    // Check the list. If its there already, unregister?
    NemesisClient c_nemesis = null;

    int i = -1;

    // Find the client based on hostname and url
    for (i = 0; i< clients.length(); i++ ) {
      if (c.getHost().equals(clients.getElement(i).getHost())) {
        // Check the url now
        if (url.getRemainder().equals(((NemesisClient)clients.getElement(i)).
          getSpider().getURL().getRemainder()))
          break;  /* Found it */
      }
    }

    if (i >= 0 && clients.length()>0) {
      c_nemesis = (NemesisClient)clients.getElement(i);
       // And what about the spider? Stop it - that stops the RTP transmission
      c_nemesis.getSpider().killThread();
    }
    else {
      c_nemesis = new NemesisClient(c.getHost(), c.getName(), dir);
      clients.addElement(c_nemesis);
      if (l != null)
        l.refresh(clients);
      if (rmi != null)
        rmi.setNrClients(rmi.getNrClients()+1);
    }
    log.println("Client "+c.getHost()+" is registering ..",c.getHost());
    try {
      log.println("Starting RTP stream to "+url.toExternalForm(),c.getHost());
      // Thread it, since it takes forever to start?
      Spider s = new Spider(log, c_nemesis,g,f, url);
      s.start();
    } catch (Exception e) {
      e.printStackTrace();
      throw new RemoteException(e.getMessage());
    }
  }
  public void unregister(MediaLocator url, Client c) throws RemoteException {

    // Find the sucker
    int i = -1;

    // Find the client based on hostname and url
    for (i = 0; i< clients.length(); i++ ) {
      if (c.getHost().equals(clients.getElement(i).getHost())) {
        // Check the url now
        if (url.getRemainder().equals(((NemesisClient)clients.getElement(i)).
          getSpider().getURL().getRemainder()))
          break;  /* Found it */
      }
    }
    if (i >= 0 && clients.length()>1) {
      NemesisClient c_nemesis = (NemesisClient)clients.getElement(i);
       // And what about the spider? Stop it - that stops the RTP transmission
      c_nemesis.getSpider().killThread();
      clients.removeElement((Client)c_nemesis);
      if (rmi != null)
        rmi.setNrClients(rmi.getNrClients()-1);
      if (l != null)
        l.refresh(clients);
    }
    log.println("Client "+c.getHost()+" is unregistering ..",c.getHost());
  }
  public void registerServerRMIInfo(ServerRMIInfo rmi) {

    this.rmi = rmi;
  }
}