package apollo.nemesis;

import apollo.nemesis.client.*;
import apollo.demeter.save.*;
import apollo.common.*;
import java.io.*;
import javax.media.*;
import javax.media.format.*;
import javax.media.protocol.*;
import javax.media.datasink.*;
import java.net.*;
import apollo.common.events.*;
/**

 * @author Konrad Rzeszutek (konrad@darnok.org)
 * @version $Id: Spider.java,v 1.2 2002/04/28 19:14:38 uid500 Exp $
 */

/**
 * Spider is responsible for checking the input of the images from the client.
 * Going over every file and checking it against the FaceRecognition. If there
 * is a match, the EventGenerator is called which prompty does whatever it
 * has to do. Otherwise this Spider continues on checking the files and
 * reading them, and deleteing.
 */

public class Spider extends Thread{

  private boolean stop = false;
  private Log log;
  private NemesisClient c;
  private NemesisEventGenerator e;
  private Object waitThread = new Object();
  private FaceRecognition f;
  private RTPtoJPG jpg_saver;
  private MediaLocator url;

  /**
   * Starts the spider.
   *
   * @param log Logging facility
   * @param c The client
   * @param e The event generator
   * @param f The face recognition engine
   * @param url Thr URL for RTP streaming.
   */
  public Spider(Log log, NemesisClient c, NemesisEventGenerator e,
      FaceRecognition f, MediaLocator url)throws MalformedURLException,
      NoProcessorException, NoDataSourceException,
      UnknownProblemException,  IOException, UnsupportedPlugInException,
      IncompatibleSourceException, UnsupportedFormatException{

    this.log = log;
    this.c = c;
    this.e = e;
    this.f = f;
    this.url = url;
    c.registerSpider(this);

    createDirectory(c.getImageDir());
    // The plug is gray -scale convertor.
    Codec[] codecs = new Codec[1];
    codecs[0] = new GrayColorEffect();
    jpg_saver = new RTPtoJPG(log, c, codecs);

    jpg_saver.open(url);
    c.registerSaver(jpg_saver);
    log.println("Started RTP "+url.toExternalForm(),c.getHost());

    // Start the RTP saver
  }
  public MediaLocator getURL() {

    return url;
  }

  /**
   * Kill the thread safely.
   */
  public void killThread() {

    jpg_saver.flush();
    jpg_saver.close();
    jpg_saver = null;
    log.println("Stopping RTP. ",c.getHost());
    stop = true;
    synchronized (waitThread) {
      try {
        waitThread.notifyAll();
       } catch (Exception er) { }
    }
    System.gc();
  }
  public void run() {

    File dir = null;
    File[] files = null;
    String[] ans = null;
    byte[] m = null;
    byte[] cam = null;

    while (!stop) {
      try {
        // Check the directory. See the amount of files
        dir = new File(c.getImageDir());
        files = dir.listFiles();
        log.println("Checking "+dir.getAbsolutePath()+", found "+files.length+
          " files. ",Log.INFO);
      // Iterate thru the files
        for (int i = 0; i< files.length; i++) {
          ans = f.submit(files[i].getAbsolutePath());

          if (ans != null) { /* There is a match */
            RandomAccessFile ra = new RandomAccessFile(ans[0],"r");
            m = new byte[(int)ra.length()];
            ra.read(m);
            ra.close();

            ra = new RandomAccessFile(files[i].getAbsolutePath(),"r");
            cam = new byte[(int)ra.length()];
            ra.read(cam);
            ra.close();

            System.out.println(m.length+" "+cam.length);
            c.setDescription(ans[0] +" matched "+files[i].getAbsolutePath()+
              ", distance: "+ans[1]);
            //log.println(c.getDescription(),Log.INFO);
            e.submit(c,cam,m);
          } //else
            files[i].delete();
       // For each file, if the distance is ... blah blah
       // call e. and with the
        }
        } catch (Exception e) {
        e.printStackTrace();
        }
       synchronized (waitThread) {
        try {
          waitThread.wait(9000);
        } catch (InterruptedException er) { }
      }
    }
  }

  /**
   * Helper method used to create directories
   *
   */
  private void createDirectory (String dir) {

          /*
        Create the directory.

        */

        File[] dirs = null;
        File top = new File(dir);
        String step = top.getAbsolutePath();
        int cnt = 0;

        for (int i = 0; i< step.length(); i++)
          if (step.charAt(i) == '\\') cnt++;

        //System.out.println(step+","+cnt);
        dirs = new File[cnt+1];
        dirs[dirs.length-1] = top;
        for (int i = dirs.length-2; i>=0; i--)
          dirs[i] = new File(dirs[i+1].getParent());

        for (int i = 0; i< dirs.length; i++) {
          //System.out.println(dirs[i].getAbsolutePath());
          dirs[i].mkdir();
        }
  }
}